# -*- coding: utf-8 -*-
# Created by restran on 2017/9/15
from __future__ import unicode_literals, absolute_import, print_function
from .. import force_text, text_type, binary_type

import itertools

__all__ = ['grouper', 'any_none', 'any_in', 'text_type_dict', 'PrintCollector', 'ObjectDict']


def grouper(iterable, size):
    """
    >>> a = grouper([1,2,3,4,5,6,7],2)
    >>> list(a)
    [(1, 2), (3, 4), (5, 6), (7,)]
    :param iterable:
    :param size:
    :return:
    """
    # http://stackoverflow.com/a/8991553
    it = iter(iterable)
    if size <= 0:
        yield it
        return
    while True:
        chunk = tuple(itertools.islice(it, size))
        if not chunk:
            return
        yield chunk


def any_none(*params):
    """
    >>> a = None
    >>> b = 'abc'
    >>> any_none(a, b)
    True
    >>> any_none(b)
    False
    """
    return any(map(lambda x: x is None, params))


def any_empty(*params):
    """
    >>> a = None
    >>> b = 'abc'
    >>> any_none(a, b)
    True
    >>> any_none(b)
    False
    """
    return any(map(lambda x: x in (None, '', False) or (iterable(x) and len(x) == 0), params))


def any_in(obj, *params):
    """
    >>> any_in(['a', ''], '')
    True
    >>> any_in(['a', None], '', None)
    True
    """
    return any(map(lambda x: x in obj, params))


def text_type_dict(dict_data):
    if not isinstance(dict_data, dict):
        raise TypeError

    new_dict = {}
    for k, v in dict_data.items():
        if isinstance(k, binary_type):
            k = k.decode('utf-8')
        if isinstance(v, binary_type):
            v = v.decode('utf-8')

        new_dict[k] = v

    return new_dict


class PrintCollector(object):
    def __init__(self):
        self.collector = []

    def print(self, output):
        self.collector.append(output)
        print(text_type(output))

    def all_output(self):
        return '\n'.join([force_text(t) for t in self.collector])

    def smart_output(self, result=None, verbose=True):
        if verbose:
            return self.all_output()

        if result is None:
            result = self.collector

        return result


class ObjectDict(dict):
    """Makes a dictionary behave like an object, with attribute-style access.
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            return None

    def __setattr__(self, name, value):
        self[name] = value


def main():
    print(any_none(1, 2, 3, None))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Created by restran on 2018/11/13
from __future__ import unicode_literals, absolute_import
from mountains import file
import unittest
import uuid
from datetime import datetime


class Test(unittest.TestCase):
    def setUp(self):
        pass

    # def test_json_loads(self):
    #     file.read_json('data/test.json')

    def test_json_dumps(self):
        pass

    # def test_file_size(self):
    #     size = file.get_file_size('data/test.txt')
    #     self.assertEqual(size, 57)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Created by restran on 2018/3/7
from __future__ import unicode_literals, absolute_import

import unittest

from mountains.encoding import converter, is_base64, is_base32


class Test(unittest.TestCase):
    def setUp(self):
        pass

    def test_hex2dec(self):
        s = '1F'
        r = converter.hex2dec(s)
        self.assertEqual(r, 31)
        s = '1F1F'
        r = converter.hex2dec(s)
        self.assertEqual(r, 7967)

    def test_dec2hex(self):
        s = '31'
        r = converter.dec2hex(s)
        self.assertEqual(r.upper(), '1F')
        s = '7967'
        r = converter.dec2hex(s)
        self.assertEqual(r.upper(), '1F1F')

    def test_bin2dec(self):
        s = '11111'
        r = converter.bin2dec(s)
        self.assertEqual(r, 31)

    def test_dec2bin(self):
        s = '31'
        r = converter.dec2bin(s)
        self.assertEqual(r, '11111')

    def test_str2dec(self):
        s = 'abcdef'
        r = converter.str2dec(s)
        self.assertEqual(r, 107075202213222)

    def test_dec2str(self):
        s = '107075202213222'
        r = converter.dec2str(s)
        self.assertEqual(r, 'abcdef')

    def test_str2hex(self):
        s = 'abcdef'
        r = converter.str2hex(s)
        self.assertEqual(r, '616263646566')

    def test_hex2str(self):
        s = '616263646566'
        r = converter.hex2str(s)
        self.assertEqual(r, 'abcdef')

    def test_hex2bin(self):
        s = '616263646566'
        r = converter.hex2bin(s)
        self.assertEqual(r, '011000010110001001100011011001000110010101100110')
        s = '00'
        r = converter.hex2bin(s)
        self.assertEqual(r, '00000000')

    def test_bin2hex(self):
        s = '011000010110001001100011011001000110010101100110'
        r = converter.bin2hex(s)
        self.assertEqual(r, '616263646566')

    def test_to_digital(self):
        for i in range(10, 1000):
            for j in range(2, 10):
                r = converter.to_digital(i, j)
                x = converter.from_digital(r, j)
                self.assertEqual(str(i), x)

    def test_is_base64(self):
        data = [
            ("Y2QgL2QgIkM6L2luZXRwdWIvd3d3cm9vdCImd2hvYW1pJmVjaG8gW1NdJmNkJmVjaG8gW0Vd", True),
            ("Qzov", True),
            ("123", False),
            ("Y21k", True),
            ("0000", False),
            ("QzovaW5ldHB1Yi93d3dyb290Lw==", True),
            ("QzovaW5ldHB1Yi8=", True),
            ("aHR0cDovLzExOC4zMS42Ni4yMy8yMDIwc2tpbGwvYS5leGU=", True)
        ]

        for (s, real_r) in data:
            r = is_base64(s)
            self.assertEqual(r, real_r)

    def test_is_base32(self):
        data = [
            ("GFQWCYLB", True),
            ("1", False),
            ("123", False),
            ("GE======", True),
            ("0000", False),
            ("GFQXGZDGMFZWIZTBONSGMYLTMRTGC43EMY======", True)
        ]

        for (s, real_r) in data:
            r = is_base32(s)
            self.assertEqual(r, real_r)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
https://github.com/kennethreitz/records

ISC License

Copyright (c) 2016, Kenneth Reitz <me@kennethreitz.org>

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""

import os
from sys import stdout
from collections import OrderedDict
from contextlib import contextmanager
from inspect import isclass

try:
    import tablib
    from docopt import docopt
    from sqlalchemy import create_engine, exc, inspect, text
except Exception as e:
    raise Exception(e)


def isexception(obj):
    """Given an object, return a boolean indicating whether it is an instance
    or subclass of :py:class:`Exception`.
    """
    if isinstance(obj, Exception):
        return True
    if isclass(obj) and issubclass(obj, Exception):
        return True
    return False


class Record(object):
    """A row, from a query, from a database."""
    __slots__ = ('_keys', '_values')

    def __init__(self, keys, values):
        self._keys = keys
        self._values = values

        # Ensure that lengths match properly.
        assert len(self._keys) == len(self._values)

    def keys(self):
        """Returns the list of column names from the query."""
        return self._keys

    def values(self):
        """Returns the list of values from the query."""
        return self._values

    def __repr__(self):
        return '<Record {}>'.format(self.export('json')[1:-1])

    def __getitem__(self, key):
        # Support for index-based lookup.
        if isinstance(key, int):
            return self.values()[key]

        # Support for string-based lookup.
        if key in self.keys():
            i = self.keys().index(key)
            if self.keys().count(key) > 1:
                raise KeyError("Record contains multiple '{}' fields.".format(key))
            return self.values()[i]

        raise KeyError("Record contains no '{}' field.".format(key))

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError as e:
            raise AttributeError(e)

    def __dir__(self):
        standard = dir(super(Record, self))
        # Merge standard attrs with generated ones (from column names).
        return sorted(standard + [str(k) for k in self.keys()])

    def get(self, key, default=None):
        """Returns the value for a given key, or default."""
        try:
            return self[key]
        except KeyError:
            return default

    def as_dict(self, ordered=False):
        """Returns the row as a dictionary, as ordered."""
        items = zip(self.keys(), self.values())

        return OrderedDict(items) if ordered else dict(items)

    @property
    def dataset(self):
        """A Tablib Dataset containing the row."""
        data = tablib.Dataset()
        data.headers = self.keys()

        row = _reduce_datetimes(self.values())
        data.append(row)

        return data

    def export(self, format, **kwargs):
        """Exports the row to the given format."""
        return self.dataset.export(format, **kwargs)


class RecordCollection(object):
    """A set of excellent Records from a query."""

    def __init__(self, rows):
        self._rows = rows
        self._all_rows = []
        self.pending = True

    def __repr__(self):
        return '<RecordCollection size={} pending={}>'.format(len(self), self.pending)

    def __iter__(self):
        """Iterate over all rows, consuming the underlying generator
        only when necessary."""
        i = 0
        while True:
            # Other code may have iterated between yields,
            # so always check the cache.
            if i < len(self):
                yield self[i]
            else:
                # Throws StopIteration when done.
                # Prevent StopIteration bubbling from generator, following https://www.python.org/dev/peps/pep-0479/
                try:
                    yield next(self)
                except StopIteration:
                    return
            i += 1

    def next(self):
        return self.__next__()

    def __next__(self):
        try:
            nextrow = next(self._rows)
            self._all_rows.append(nextrow)
            return nextrow
        except StopIteration:
            self.pending = False
            raise StopIteration('RecordCollection contains no more rows.')

    def __getitem__(self, key):
        is_int = isinstance(key, int)

        # Convert RecordCollection[1] into slice.
        if is_int:
            key = slice(key, key + 1)

        while len(self) < key.stop or key.stop is None:
            try:
                next(self)
            except StopIteration:
                break

        rows = self._all_rows[key]
        if is_int:
            return rows[0]
        else:
            return RecordCollection(iter(rows))

    def __len__(self):
        return len(self._all_rows)

    def export(self, format, **kwargs):
        """Export the RecordCollection to a given format (courtesy of Tablib)."""
        return self.dataset.export(format, **kwargs)

    @property
    def dataset(self):
        """A Tablib Dataset representation of the RecordCollection."""
        # Create a new Tablib Dataset.
        data = tablib.Dataset()

        # If the RecordCollection is empty, just return the empty set
        # Check number of rows by typecasting to list
        if len(list(self)) == 0:
            return data

        # Set the column names as headers on Tablib Dataset.
        first = self[0]

        data.headers = first.keys()
        for row in self.all():
            row = _reduce_datetimes(row.values())
            data.append(row)

        return data

    def all(self, as_dict=False, as_ordereddict=False):
        """Returns a list of all rows for the RecordCollection. If they haven't
        been fetched yet, consume the iterator and cache the results."""

        # By calling list it calls the __iter__ method
        rows = list(self)

        if as_dict:
            return [r.as_dict() for r in rows]
        elif as_ordereddict:
            return [r.as_dict(ordered=True) for r in rows]

        return rows

    def as_dict(self, ordered=False):
        return self.all(as_dict=not (ordered), as_ordereddict=ordered)

    def first(self, default=None, as_dict=False, as_ordereddict=False):
        """Returns a single record for the RecordCollection, or `default`. If
        `default` is an instance or subclass of Exception, then raise it
        instead of returning it."""

        # Try to get a record, or return/raise default.
        try:
            record = self[0]
        except IndexError:
            if isexception(default):
                raise default
            return default

        # Cast and return.
        if as_dict:
            return record.as_dict()
        elif as_ordereddict:
            return record.as_dict(ordered=True)
        else:
            return record

    def one(self, default=None, as_dict=False, as_ordereddict=False):
        """Returns a single record for the RecordCollection, ensuring that it
        is the only record, or returns `default`. If `default` is an instance
        or subclass of Exception, then raise it instead of returning it."""

        # Ensure that we don't have more than one row.
        try:
            self[1]
        except IndexError:
            return self.first(default=default, as_dict=as_dict, as_ordereddict=as_ordereddict)
        else:
            raise ValueError('RecordCollection contained more than one row. '
                             'Expects only one row when using '
                             'RecordCollection.one')

    def scalar(self, default=None):
        """Returns the first column of the first row, or `default`."""
        row = self.one()
        return row[0] if row else default


class Database(object):
    """A Database. Encapsulates a url and an SQLAlchemy engine with a pool of
    connections.
    """

    def __init__(self, db_url=None, **kwargs):
        # If no db_url was provided, fallback to $DATABASE_URL.
        self.db_url = db_url or os.environ.get('DATABASE_URL')

        if not self.db_url:
            raise ValueError('You must provide a db_url.')

        # Create an engine.
        self._engine = create_engine(self.db_url, **kwargs)
        self.open = True

    def close(self):
        """Closes the Database."""
        self._engine.dispose()
        self.open = False

    def __enter__(self):
        return self

    def __exit__(self, exc, val, traceback):
        self.close()

    def __repr__(self):
        return '<Database open={}>'.format(self.open)

    def get_table_names(self, internal=False):
        """Returns a list of table names for the connected database."""

        # Setup SQLAlchemy for Database inspection.
        return inspect(self._engine).get_table_names()

    def get_connection(self, close_with_result=False):
        """Get a connection to this Database. Connections are retrieved from a
        pool.
        """
        if not self.open:
            raise exc.ResourceClosedError('Database closed.')

        return Connection(self._engine.connect(close_with_result=close_with_result), close_with_result)

    def query(self, query, fetchall=False, **params):
        """Executes the given SQL query against the Database. Parameters can,
        optionally, be provided. Returns a RecordCollection, which can be
        iterated over to get result rows as dictionaries.
        """
        with self.get_connection(True) as conn:
            return conn.query(query, fetchall, **params)

    def bulk_query(self, query, *multiparams):
        """Bulk insert or update."""

        with self.get_connection(True) as conn:
            conn.bulk_query(query, *multiparams)

    def query_file(self, path, fetchall=False, **params):
        """Like Database.query, but takes a filename to load a query from."""

        with self.get_connection(True) as conn:
            return conn.query_file(path, fetchall, **params)

    def bulk_query_file(self, path, *multiparams):
        """Like Database.bulk_query, but takes a filename to load a query from."""

        with self.get_connection() as conn:
            conn.bulk_query_file(path, *multiparams)

    @contextmanager
    def transaction(self):
        """A context manager for executing a transaction on this Database."""

        conn = self.get_connection()
        tx = conn.transaction()
        try:
            yield conn
            tx.commit()
        except:
            tx.rollback()
        finally:
            conn.close()


class Connection(object):
    """A Database connection."""

    def __init__(self, connection, close_with_result=False):
        self._conn = connection
        self.open = not connection.closed
        self._close_with_result = close_with_result

    def close(self):
        # No need to close if this connection is used for a single result.
        # The connection will close when the results are all consumed or GCed.
        if not self._close_with_result:
            self._conn.close()
        self._conn.close()
        self.open = False

    def __enter__(self):
        return self

    def __exit__(self, exc, val, traceback):
        self.close()

    def __repr__(self):
        return '<Connection open={}>'.format(self.open)

    def query(self, query, fetchall=False, **params):
        """Executes the given SQL query against the connected Database.
        Parameters can, optionally, be provided. Returns a RecordCollection,
        which can be iterated over to get result rows as dictionaries.
        """

        # Execute the given query.
        cursor = self._conn.execute(text(query), **params)  # TODO: PARAMS GO HERE

        # Row-by-row Record generator.
        row_gen = iter(Record([], []))

        if cursor.returns_rows:
            row_gen = (Record(cursor.keys(), row) for row in cursor)

        # Convert psycopg2 results to RecordCollection.
        results = RecordCollection(row_gen)

        # Fetch all results if desired.
        if fetchall:
            results.all()

        return results

    def bulk_query(self, query, *multiparams):
        """Bulk insert or update."""

        self._conn.execute(text(query), *multiparams)

    def query_file(self, path, fetchall=False, **params):
        """Like Connection.query, but takes a filename to load a query from."""

        # If path doesn't exists
        if not os.path.exists(path):
            raise IOError("File '{}' not found!".format(path))

        # If it's a directory
        if os.path.isdir(path):
            raise IOError("'{}' is a directory!".format(path))

        # Read the given .sql file into memory.
        with open(path) as f:
            query = f.read()

        # Defer processing to self.query method.
        return self.query(query=query, fetchall=fetchall, **params)

    def bulk_query_file(self, path, *multiparams):
        """Like Connection.bulk_query, but takes a filename to load a query
        from.
        """

        # If path doesn't exists
        if not os.path.exists(path):
            raise IOError("File '{}'' not found!".format(path))

        # If it's a directory
        if os.path.isdir(path):
            raise IOError("'{}' is a directory!".format(path))

        # Read the given .sql file into memory.
        with open(path) as f:
            query = f.read()

        self._conn.execute(text(query), *multiparams)

    def transaction(self):
        """Returns a transaction object. Call ``commit`` or ``rollback``
        on the returned object as appropriate."""

        return self._conn.begin()


def _reduce_datetimes(row):
    """Receives a row, converts datetimes to strings."""

    row = list(row)

    for i in range(len(row)):
        if hasattr(row[i], 'isoformat'):
            row[i] = row[i].isoformat()
    return tuple(row)


def cli():
    supported_formats = 'csv tsv json yaml html xls xlsx dbf latex ods'.split()
    formats_lst = ", ".join(supported_formats)
    cli_docs = """Records: SQL for Humans™
A Kenneth Reitz project.

Usage:
  records <query> [<format>] [<params>...] [--url=<url>]
  records (-h | --help)

Options:
  -h --help     Show this screen.
  --url=<url>   The database URL to use. Defaults to $DATABASE_URL.

Supported Formats:
   %(formats_lst)s

   Note: xls, xlsx, dbf, and ods formats are binary, and should only be
         used with redirected output e.g. '$ records sql xls > sql.xls'.

Query Parameters:
    Query parameters can be specified in key=value format, and injected
    into your query in :key format e.g.:

    $ records 'select * from repos where language ~= :lang' lang=python

Notes:
  - While you may specify a database connection string with --url, records
    will automatically default to the value of $DATABASE_URL, if available.
  - Query is intended to be the path of a SQL file, however a query string
    can be provided instead. Use this feature discernfully; it's dangerous.
  - Records is intended for report-style exports of database queries, and
    has not yet been optimized for extremely large data dumps.
    """ % dict(formats_lst=formats_lst)

    # Parse the command-line arguments.
    arguments = docopt(cli_docs)

    query = arguments['<query>']
    params = arguments['<params>']
    format = arguments.get('<format>')
    if format and "=" in format:
        del arguments['<format>']
        arguments['<params>'].append(format)
        format = None
    if format and format not in supported_formats:
        print('%s format not supported.' % format)
        print('Supported formats are %s.' % formats_lst)
        exit(62)

    # Can't send an empty list if params aren't expected.
    try:
        params = dict([i.split('=') for i in params])
    except ValueError:
        print('Parameters must be given in key=value format.')
        exit(64)

    # Be ready to fail on missing packages
    try:
        # Create the Database.
        db = Database(arguments['--url'])

        # Execute the query, if it is a found file.
        if os.path.isfile(query):
            rows = db.query_file(query, **params)

        # Execute the query, if it appears to be a query string.
        elif len(query.split()) > 2:
            rows = db.query(query, **params)

        # Otherwise, say the file wasn't found.
        else:
            print('The given query could not be found.')
            exit(66)

        # Print results in desired format.
        if format:
            content = rows.export(format)
            if isinstance(content, bytes):
                print_bytes(content)
            else:
                print(content)
        else:
            print(rows.dataset)
    except ImportError as impexc:
        print(impexc.msg)
        print("Used database or format require a package, which is missing.")
        print("Try to install missing packages.")
        exit(60)


def print_bytes(content):
    try:
        stdout.buffer.write(content)
    except AttributeError:
        stdout.write(content)


# Run the CLI when executed directly.
if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
# created by restran on 2019/07/12
from __future__ import unicode_literals, absolute_import
import math

try:
    from . import records
except Exception as e:
    raise Exception('records is not installed')


def batch_insert(db, sql, data, batch_size):
    """
    :type db: records.Database
    :param db:
    :param sql:
    :param data:
    :param batch_size:
    :return:
    """

    total = len(data)
    if total <= 0:
        return

    times = math.ceil(total * 1.0 / batch_size)
    for i in range(times):
        data_list = data[i * batch_size:(i + 1) * batch_size]
        if len(data_list) > 0:
            db.bulk_query(sql, data_list)


def auto_batch_insert(db, sql, data, batch_size=500, force=False):
    """
    可以边添加数据，边自动批量插入数据
    :type db: records.Database
    :param db:
    :param sql:
    :param data:
    :param batch_size:
    :param force:
    :return:
    """
    total = len(data)
    if total <= 0:
        return data

    if total >= batch_size or force:
        if len(data) > 0:
            db.bulk_query(sql, data)
        data = []

    return data



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

import time
from concurrent import futures

from ..base import Queue


class TaskExecutor(object):
    """
    使用线程的执行器，可以并发执行任务
    """

    def __init__(self, fn_task, task_params_list, max_workers=5):
        self.fn_task = fn_task
        self.max_workers = max_workers
        self.task_list = task_params_list
        self.task_queue = Queue()
        for t in task_params_list:
            self.task_queue.put(t)

    def get_next_tasks(self, max_num):
        output = []
        count = 0
        while not self.task_queue.empty() and count < max_num:
            t = self.task_queue.get()
            output.append(t)
            count += 1

        return output

    def run(self, *args, **kwargs):
        print('executor start')
        start_time = time.time()
        with futures.ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            next_tasks = self.get_next_tasks(self.max_workers)
            shut_down = False
            while not shut_down and len(next_tasks) > 0:
                future_to_task = {
                    executor.submit(self.fn_task, task, *args, **kwargs): task
                    for task in next_tasks
                }

                # 这里 ThreadPoolExecutor 必须要等当前的所有任务都执行完成后，
                # 才能开始下一批的任务
                for future in futures.as_completed(future_to_task):
                    _ = future_to_task[future]
                    try:
                        shut_down = future.result()
                    except Exception as exc:
                        print(exc)
                        continue

                    if shut_down:
                        break

                next_tasks = self.get_next_tasks(self.max_workers)
        end_time = time.time()
        print('executor done, %.3fs' % (end_time - start_time))



# -*- coding: utf-8 -*-
# Created by restran on 2018/3/7
from __future__ import unicode_literals, absolute_import

import binascii
import struct
from xml.sax.saxutils import escape as xml_escape_func
from xml.sax.saxutils import unescape as xml_unescape_func

from mountains.encoding import force_bytes, force_text


def to_uu(data):
    """
    uu编码
    :param data: 字符串
    :return: 编码后的字符串
    """
    r = binascii.b2a_uu(force_bytes(data))
    return force_text(r)


def from_uu(data):
    """
    解uu编码
    :param data: uu编码的字符串
    :return: 字符串
    """
    r = binascii.a2b_uu(data)
    return force_text(r)


def str2hex(s):
    """
    把一个字符串转成其ASCII码的16进制表示
    :param s: 要转换的字符串
    :return: ASCII码的16进制表示字符串
    """
    return force_text(binascii.b2a_hex(force_bytes(s)))


def hex2str(s):
    """
    把十六进制字符串转换成其ASCII表示字符串
    :param s: 十六进制字符串
    :return: 字符串
    """
    return force_text(binascii.a2b_hex(s))


base = [str(x) for x in range(10)] + [chr(x) for x in range(ord('A'), ord('A') + 6)]


def bin2dec(s):
    """
    bin2dec
    二进制 to 十进制: int(str,n=10)
    :param s:
    :return:
    """
    return int(s, 2)


def dec2bin(s):
    """
    dec2bin
    十进制 to 二进制: bin()
    :param s:
    :return:
    """
    if not isinstance(s, int):
        num = int(s)
    else:
        num = s
    mid = []
    while True:
        if num == 0:
            break
        num, rem = divmod(num, 2)
        mid.append(base[rem])

    return ''.join([str(x) for x in mid[::-1]])


def hex2dec(s):
    """
    hex2dec
    十六进制 to 十进制
    :param s:
    :return:
    """
    if not isinstance(s, str):
        s = str(s)
    return int(s.upper(), 16)


def dec2hex(s):
    """
    dec2hex
    十进制 to 八进制: oct()
    十进制 to 十六进制: hex()
    :param s:
    :return:
    """
    if not isinstance(s, int):
        num = int(s)
    else:
        num = s
    return hex(num)[2:]


def byte2hex(s):
    """
    字节类型的数据转成16进制字符串
    :param s:
    :return:
    """

    result = binascii.b2a_hex(s)
    return force_text(result)


def hex2bin(s):
    """
    hex2tobin
    十六进制 to 二进制: bin(int(str,16))
    :param s:
    :return:
    """
    if len(s) % 2 != 0:
        s = '0' + s

    result = []
    for i in range(len(s) // 2):
        t = s[i * 2:(i + 1) * 2]
        x = dec2bin(hex2dec(t.upper()))
        padding_length = (8 - len(x) % 8) % 8
        # 每个16进制值（2个字符）进行转码，不足8个的，在前面补0
        x = '%s%s' % ('0' * padding_length, x)
        result.append(x)

    result = ''.join(result)
    return result if result != '' else '00000000'


def bin2hex(s):
    """
    bin2hex
    二进制 to 十六进制: hex(int(str,2))
    :param s:
    :return:
    """
    padding_length = (8 - len(s) % 8) % 8
    # 从前往后解码，不足8个的，在后面补0
    encode_str = '%s%s' % (s, '0' * padding_length)
    # 解码后是 0xab1234，需要去掉前面的 0x
    return hex(int(encode_str, 2))[2:].rstrip('L')


def str2dec(s):
    """
    string to decimal number.
    """
    if not len(s):
        return 0
    return int(str2hex(s), 16)


def dec2str(n):
    """
    decimal number to string.
    """
    s = hex(int(n))[2:].rstrip('L')
    if len(s) % 2 != 0:
        s = '0' + s
    return hex2str(s)


def str2bin(s):
    """
    String to binary.
    """
    ret = []
    for c in s:
        ret.append(bin(ord(c))[2:].zfill(8))
    return ''.join(ret)


def bin2str(b):
    """
    Binary to string.
    """
    ret = []
    for pos in range(0, len(b), 8):
        ret.append(chr(int(b[pos:pos + 8], 2)))
    return ''.join(ret)


def from_digital(s, num):
    """
    进制转换，从指定机制转到10进制
    :param s:
    :param num:
    :return:
    """
    if not 1 < num < 10:
        raise ValueError('digital num must between 1 and 10')
    return '%s' % int(s, num)


def to_digital(d, num):
    """
    进制转换，从10进制转到指定机制
    :param d:
    :param num:
    :return:
    """
    if not isinstance(num, int) or not 1 < num < 10:
        raise ValueError('digital num must between 1 and 10')

    d = int(d)
    result = []
    x = d % num
    d = d - x
    result.append(str(x))
    while d > 0:
        d = d // num
        x = d % num
        d = d - x
        result.append(str(x))
    return ''.join(result[::-1])


def xml_escape(data):
    return xml_escape_func(data)


def xml_un_escape(data):
    return xml_unescape_func(data)


def str2int(number_str, default_value=None):
    if number_str is None or number_str == '':
        return default_value
    try:
        return int(number_str)
    except Exception as e:
        return default_value


def long2bytes(n, block_size=0):
    """Convert an integer to a byte string.

    In Python 3.2+, use the native method instead::

        >>> n.to_bytes(block_size, 'big')

    For instance::

        >>> n = 80
        >>> n.to_bytes(2, 'big')
        b'\x00P'

    If the optional :data:`blocksize` is provided and greater than zero,
    the byte string is padded with binary zeros (on the front) so that
    the total length of the output is a multiple of blocksize.

    If :data:`blocksize` is zero or not provided, the byte string will
    be of minimal length.
    """
    # after much testing, this algorithm was deemed to be the fastest
    s = b''
    n = int(n)
    pack = struct.pack
    while n > 0:
        s = pack('>I', n & 0xffffffff) + s
        n = n >> 32
    # strip off leading zeros
    for i in range(len(s)):
        if s[i] != b'\000'[0]:
            break
    else:
        # only happens when n == 0
        s = b'\000'
        i = 0
    s = s[i:]
    # add back some pad bytes.  this could be done more efficiently w.r.t. the
    # de-padding being done above, but sigh...
    if block_size > 0 and len(s) % block_size:
        s = (block_size - len(s) % block_size) * b'\000' + s
    return s


def bytes2long(s):
    """Convert a byte string to a long integer (big endian).

    In Python 3.2+, use the native method instead::

        >>> int.from_bytes(s, 'big')

    For instance::

        >>> int.from_bytes(b'\x00P', 'big')
        80

    This is (essentially) the inverse of :func:`long_to_bytes`.
    """
    acc = 0
    unpack = struct.unpack
    length = len(s)
    if length % 4:
        extra = (4 - length % 4)
        s = b'\000' * extra + s
        length = length + extra
    for i in range(0, length, 4):
        acc = (acc << 32) + unpack('>I', s[i:i + 4])[0]
    return acc



# -*- coding: utf-8 -*-
# Created by restran on 2017/10/13
from __future__ import unicode_literals, absolute_import

import logging
import unittest

from mountains.utils import any_in, any_none

logger = logging.getLogger(__name__)


class UtilsTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_any_none(self):
        self.assertEqual(any_none(1, 2, 3, None), True)
        self.assertEqual(any_none(1, 2, 3), False)

    def test_any_in(self):
        self.assertEqual(any_in('123abc123', 'a', 'b'), True)
        self.assertEqual(any_in('123ab123', 'a', 'c'), True)
        self.assertEqual(any_in('123', 'a', 'b', 'c'), False)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Created by restran on 2017/10/13
from __future__ import unicode_literals, absolute_import

import logging

from .. import json

try:
    from django.http import HttpResponse
except:
    class HttpResponse(object):
        def __init__(*args, **kwargs):
            raise Exception('django is not installed')

logger = logging.getLogger(__name__)


def http_response_json(dict_data):
    """
    返回json数据
    :param dict_data:
    :return:
    """

    return HttpResponse(json.dumps(dict_data),
                        content_type="application/json; charset=utf-8")


class APIStatusCode(object):
    SUCCESS = 200  # 成功
    FAIL = 400  # 客户端的错误, 例如请求信息不正确
    ERROR = 500  # 服务端的错误, 例如出现异常
    LOGIN_REQUIRED = 401  # 需要登录才能访问


class APIHandler(object):
    @classmethod
    def return_json(cls, code, data, msg):
        try:
            return http_response_json({
                'code': code, 'data': data, 'msg': msg})
        except Exception as e:
            logger.error(e)
            return http_response_json({
                'code': APIStatusCode.ERROR, 'data': None,
                'msg': msg})

    @classmethod
    def success(cls, data=None, msg='', code=APIStatusCode.SUCCESS):
        return cls.return_json(code, data, msg)

    @classmethod
    def fail(cls, data=None, msg='', code=APIStatusCode.FAIL):
        return cls.return_json(code, data, msg)

    @classmethod
    def login_required(cls, data=None, msg='', code=APIStatusCode.LOGIN_REQUIRED):
        return cls.return_json(code, data, msg)

    @classmethod
    def error(cls, data=None, msg='', code=APIStatusCode.ERROR):
        return cls.return_json(code, data, msg)



# -*- coding: utf-8 -*-
# created by restran on 2018/01/22
from __future__ import unicode_literals, absolute_import

import time
import unittest
from datetime import datetime
from mountains.datetime import converter


class DateTimeConverterTest(unittest.TestCase):
    def setUp(self):
        self.date_str = '2016-10-30 12:30:30'
        self.dt = datetime(year=2016, month=10, day=30, hour=12, minute=30, second=30)
        self.t = self.dt.timetuple()
        self.ts = int(time.mktime(self.t))
        self.ts_ms = int(time.mktime(self.t) * 1000)

    def test_str2datetime(self):
        dt = converter.str2datetime(self.date_str)
        self.assertEqual(dt, self.dt)

    def test_str2time(self):
        t = converter.str2time(self.date_str)
        self.assertEqual(t[0:6], self.t[0:6])

    def test_str2timestamp(self):
        ts = converter.str2timestamp(self.date_str)
        self.assertEqual(ts, self.ts)
        ts = converter.str2timestamp(self.date_str, millisecond=True)
        self.assertEqual(ts, self.ts_ms)

    def test_datetime2str(self):
        s = converter.datetime2str(self.dt)
        self.assertEqual(s, self.date_str)

    def test_datetime2time(self):
        t = converter.datetime2time(self.dt)
        self.assertEqual(t[0:6], self.t[0:6])

    def test_datetime2timestamp(self):
        ts = converter.datetime2timestamp(self.dt)
        self.assertEqual(ts, self.ts)
        ts = converter.datetime2timestamp(self.dt, millisecond=True)
        self.assertEqual(ts, self.ts_ms)

    def test_time2str(self):
        s = converter.time2str(self.t)
        self.assertEqual(s, self.date_str)

    def test_time2datetime(self):
        dt = converter.time2datetime(self.t)
        self.assertEqual(dt, self.dt)

    def test_time2timestamp(self):
        ts = converter.time2timestamp(self.t)
        self.assertEqual(ts, self.ts)
        ts = converter.time2timestamp(self.t, millisecond=True)
        self.assertEqual(ts, self.ts_ms)

    def test_timestamp2datetime(self):
        dt = converter.timestamp2datetime(self.ts)
        self.assertEqual(dt, self.dt)
        dt = converter.timestamp2datetime(self.ts_ms, millisecond=True)
        self.assertEqual(dt, self.dt)

    def test_timestamp2time(self):
        t = converter.timestamp2time(self.ts)
        self.assertEqual(t[0:6], self.t[0:6])
        t = converter.timestamp2time(self.ts_ms, millisecond=True)
        self.assertEqual(t[0:6], self.t[0:6])

    def test_timestamp2str(self):
        s = converter.timestamp2str(self.ts)
        self.assertEqual(s, self.date_str)
        s = converter.timestamp2str(self.ts_ms, millisecond=True)
        self.assertEqual(s, self.date_str)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# created by restran on 2018/04/16
from __future__ import unicode_literals, absolute_import

from datetime import datetime

from ..base import BytesIO, text_type, iteritems
from ..file import write_bytes_file

__all__ = ['read_excel', 'write_excel', 'edit_excel', 'to_excel_column_no']


def read_excel(file_name=None, file_contents=None, offset=1,
               header_index=0, sheet_index=0, sheet_name=None,
               dt2str=True, lower_header=True):
    """
    读取 Excel
    :param sheet_name:
    :param dt2str: 将日期类型的数据转成字符串
    :param header_index: header 在哪一行
    :param file_contents:
    :param sheet_index:
    :param file_name:
    :param offset: 偏移，一般第一行是表头，不需要读取数据
    :param lower_header: 标题转成小写
    :return:
    """
    try:
        import xlrd
        from xlrd import xldate_as_tuple
    except:
        raise Exception('xlrd is not installed')

    try:
        workbook = xlrd.open_workbook(filename=file_name, file_contents=file_contents)
    except Exception as e:
        return None

    if len(workbook.sheets()) <= 0:
        return []

    if sheet_name is not None:
        sh = workbook.sheet_by_name(sheet_name)
    else:
        sh = workbook.sheet_by_index(sheet_index)

    raw_data = []
    n_rows = sh.nrows
    row = sh.row_values(header_index)
    header = []
    for t in row:
        t = text_type(t).strip()
        if lower_header:
            t = t.lower()
        header.append(t)

    # n_cols = sh.ncols
    # 第0行是提示信息和标题，跳过
    for i in range(offset, n_rows):
        try:
            # row = sh.row_values(i)
            d = {}
            # ctype: 0 empty,1 string, 2 number, 3 date, 4 boolean, 5 error
            for j, t in enumerate(header):
                ctype = sh.cell(i, j).ctype  # 表格的数据类型
                cell = sh.cell_value(i, j)
                if ctype == 2 and cell % 1 == 0:  # 如果是整形
                    cell = int(cell)
                elif ctype == 3:
                    # 转成datetime对象
                    cell = datetime(*xldate_as_tuple(cell, 0))
                    if dt2str:
                        cell = cell.strftime('%Y-%m-%d %H:%M:%S')
                elif ctype == 4:
                    cell = True if cell == 1 else False

                d[t] = cell
            raw_data.append(d)
        except Exception as e:
            pass

    return raw_data


def write_excel(headers, data, file_name, file_io=None):
    """
    写数据到新的Excel中
    :param headers:
    :param data:
    :param file_name:
    :param file_io:
    :return:
    """
    try:
        import xlsxwriter
    except:
        raise Exception('xlsxwriter is not installed')

    sio = BytesIO()
    workbook = xlsxwriter.Workbook(sio)
    worksheet = workbook.add_worksheet()
    new_headers = []
    if len(headers) > 0:
        for t in headers:
            if isinstance(t, dict):
                new_headers.append(t.get('name', ''))
            else:
                new_headers.append(t)
    headers = new_headers
    for i, t in enumerate(headers):
        worksheet.write(0, i, t)

    index = 1
    for row in data:
        if isinstance(row, dict):
            for i, name in enumerate(headers):
                worksheet.write(index, i, row.get(name, ''))
        else:
            for i, x in enumerate(row):
                worksheet.write(index, i, x)
        index += 1
    # 关闭 Excel
    workbook.close()
    if file_io is not None:
        if not isinstance(file_io, BytesIO):
            raise Exception('output_fio should be BytesIO')
        else:
            file_io.write(sio.getvalue())
    else:
        write_bytes_file(file_name, sio.getvalue())


def edit_excel(file_name=None, sheet_index=0, sheet_name=None, data=None, output_filename=None, output_fio=None):
    """
    编辑 Excel，打开已有的 Excel，往里面填充数据
    :param file_name:
    :param sheet_index:
    :param sheet_name:
    :param data: data = {'A2': '123', 'A3': '456'}
    :param output_filename:
    :param output_fio:
    :type data: dict
    """
    try:
        from openpyxl import load_workbook
    except:
        raise Exception('openpyxl is not installed')

    try:
        wb = load_workbook(file_name)
        if sheet_name is not None:
            ws = wb[sheet_name]
        else:
            ws = wb.worksheets[sheet_index]
    except Exception as e:
        return None

    for key, value in iteritems(data):
        try:
            ws[key] = value
        except:
            pass

    if output_fio is not None:
        if not isinstance(output_fio, BytesIO):
            raise Exception('output_fio should be BytesIO')
        else:
            wb.save(output_fio)
    elif output_filename is not None:
        wb.save(output_filename)
    else:
        wb.save(file_name)

    wb.close()


def to_excel_column_no(index, index_start_1=False):
    """
    Excel 列编号是: A, B, ..., AA, ..., BA
    index 编号默认从1开始
    """
    if index_start_1:
        index -= 1
    y = ''
    s = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'

    while index >= 0:
        y += s[index % 26]
        index = index // 26 - 1

    return y[::-1]



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

from mountains.concurrent import TaskExecutor
import time
import random
import unittest


class DateTimeConverterTest(unittest.TestCase):

    def test_task_executor(self):
        def fn_task(item):
            print(item)
            time.sleep(random.choice(range(0, 10)) / 10.0)

        task_params_list = range(3)
        t = TaskExecutor(fn_task, task_params_list, max_workers=2)
        t.run()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

import os
import random

import requests

from ..base import __base_path
from ..file import read_dict
from ..encoding import force_bytes

DEFAULT_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2983.0 Safari/537.36'

GLOBAL_USER_AGENTS = {}

USER_AGENT_DATA_PATH = os.path.join(__base_path, 'http/data/user_agents.txt')
MOBILE_USER_AGENT_DATA_PATH = os.path.join(__base_path, 'http/data/mobile_ua.txt')


def random_agent(agent_type='pc'):
    """
    随机获取一个 User-Agent
    :return:
    """
    agent_type = agent_type.lower().strip()
    if agent_type in ('wexin', 'wx'):
        agent_type = 'wechat'

    if agent_type in ('ios',):
        agent_type = 'iphone'

    global GLOBAL_USER_AGENTS
    if agent_type not in GLOBAL_USER_AGENTS:
        if agent_type == 'pc':
            GLOBAL_USER_AGENTS[agent_type] = read_dict(USER_AGENT_DATA_PATH)
        elif agent_type in ('mobile', 'wechat', 'android', 'iphone', 'alipay'):
            if 'mobile' not in GLOBAL_USER_AGENTS:
                GLOBAL_USER_AGENTS['mobile'] = list(read_dict(MOBILE_USER_AGENT_DATA_PATH))
            mobile_data = GLOBAL_USER_AGENTS['mobile']

            if agent_type == 'wechat':
                GLOBAL_USER_AGENTS[agent_type] = [t for t in mobile_data if 'MicroMessenger' in t]
            elif agent_type == 'alipay':
                GLOBAL_USER_AGENTS[agent_type] = [t for t in mobile_data if 'Alipay' in t]
            elif agent_type == 'android':
                GLOBAL_USER_AGENTS[agent_type] = [t for t in mobile_data if 'Android' in t]
            elif agent_type == 'iphone':
                GLOBAL_USER_AGENTS[agent_type] = [t for t in mobile_data if 'iPhone' in t]
            else:
                GLOBAL_USER_AGENTS[agent_type] = mobile_data
        else:
            return DEFAULT_USER_AGENT

    return random.choice(GLOBAL_USER_AGENTS[agent_type])


def random_wx_agent():
    """
    返回一个微信的UserAgent
    :return:
    """
    return random_agent('wechat')


def random_mobile_agent():
    """
    返回一个手机端的UserAgent
    :return:
    """
    return random_agent('mobile')


def request(method, url, headers=None, data=None, session=None):
    """
    :type session requests.session
    :param method:
    :param url:
    :param headers:
    :param data:
    :param session:
    :return:
    """
    base_headers = {
        'User-Agent': random_agent()
    }
    if headers is None:
        headers = {}

    base_headers.update(headers)

    if 'Content-Length' in headers:
        del base_headers['Content-Length']

    headers = base_headers
    if session is not None:
        req = session.request
    else:
        req = requests.request

    r = req(method, url, headers=headers, data=data)
    return r


def read_request(file_name, **params):
    """
    从文件中读取请求头，并根据格式化字符串模板，进行字符串格式化
    :param file_name:
    :param params:
    :return:
    """
    with open(file_name, 'r') as f:
        data = f.read()
        return read_request_from_str(data, **params)


def read_request_from_str(data, **params):
    """
    从字符串中读取请求头，并根据格式化字符串模板，进行字符串格式化
    :param data:
    :param params:
    :return:
    """
    method, uri = None, None
    headers = {}
    host = ''

    try:
        split_list = data.split('\n\n')
        headers_text = split_list[0]
        body = '\n\n'.join(split_list[1:])
    except:
        headers_text = data
        body = ''

    body = force_bytes(body)
    for k, v in params.items():
        body = body.replace(b'{%s}' % force_bytes(k), force_bytes(v))

    header_list = headers_text.split('\n')

    for i, line in enumerate(header_list):
        line = line.strip()
        if line.strip() == '':
            continue

        line = line.format(**params)
        if i == 0:
            # 至多3个
            split_line = line.strip().split(' ')
            method, uri, _ = split_line[0], ' '.join(split_line[1:-1]), split_line[-1]
        else:
            # 至多2个
            header, value = line.split(':', 1)
            header = header.strip()
            value = value.strip()
            headers[header] = value
            if header.lower() == 'host':
                host = value

    return headers, method, uri, host, body


def query_str_2_dict(query_str):
    """
    将查询字符串，转换成字典
    a=123&b=456
    {'a': '123', 'b': '456'}
    :param query_str:
    :return:
    """
    if query_str:
        query_list = query_str.split('&')
        query_dict = {}
        for t in query_list:
            # 避免在非法查询字符串的情况下报错
            # 例如未对url参数转义
            try:
                x = t.split('=')
                query_dict[x[0]] = x[1]
            except:
                pass
    else:
        query_dict = {}
    return query_dict


def raw_headers_to_dict(raw_headers):
    """
    通过原生请求头获取请求头字典
    :param raw_headers: {str} 浏览器请求头
    :return: {dict} headers
    """
    return dict(line.split(": ", 1) for line in raw_headers.split("\n") if ': ' in line)



# -*- coding: utf-8 -*-
# Created by restran on 2018/6/5
from __future__ import unicode_literals, absolute_import

import logging
import time
from ..encoding import force_text

try:
    import paramiko
except ImportError:
    raise Exception('paramiko is not installed')

logger = logging.getLogger(__name__)


class SSHClient(object):
    def __init__(self, host, port, username, password=None, key_file=None,
                 key_pass=None, show_output=False, manual_connect=False, timeout=10):
        """

        :param host:
        :param port:
        :param username:
        :param password:
        :param key_file:
        :param key_pass:
        :param show_output: 是否显示命令的执行结果
        :param manual_connect:
        :param timeout:
        """
        self.is_root = False
        self.host = host
        self.port = int(port)
        self.username = username
        self.password = password
        self.ssh_session = paramiko.SSHClient()
        self.ssh_session.load_system_host_keys()
        self.key_file = key_file
        self.key_pass = key_pass
        self.ssh_session.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.private_key = None
        self.sftp = None
        self.show_output = show_output
        self.timeout = timeout
        if not manual_connect:
            use_key = self.key_file is not None
            self.ssh_connect(password, use_key)

    def ssh_connect(self, password=None, use_key=False):
        if not use_key:
            if password is None:
                password = self.password

            self.ssh_session.connect(hostname=self.host, port=self.port,
                                     username=self.username, password=password,
                                     look_for_keys=False, timeout=self.timeout)
        else:
            self.private_key = paramiko.RSAKey.from_private_key_file(
                self.key_file, self.key_pass)
            self.ssh_session.connect(hostname=self.host, port=self.port,
                                     username=self.username, pkey=self.private_key,
                                     timeout=self.timeout)

    def get_sftp(self, refresh=False):
        if self.sftp is None or refresh:
            self.sftp = paramiko.SFTPClient.from_transport(self.ssh_session.get_transport())

        return self.sftp

    def dispose(self):
        try:
            if self.sftp is not None:
                self.sftp.close()
        except:
            pass

        try:
            if self.ssh_session is not None:
                self.ssh_session.close()
        except:
            pass

    @classmethod
    def clean_ssh_line_output(cls, line):
        line = line.strip()
        split_list = [t for t in line.split(' ') if t != '']
        return split_list

    def run(self, cmd):
        stdin, stdout, stderr = self.ssh_session.exec_command(cmd)
        # stdin这个是输入的命令，stdout这个是命令的正确返回，stderr这个是命令的错误返回
        out = stdout.readlines()
        err = stderr.readlines()
        result = []
        if isinstance(out, list):
            result.extend(out)
        if isinstance(err, list):
            result.extend(err)

        if self.show_output:
            logger.info(''.join(result))
        else:
            logger.debug(''.join(result))
        return ''.join(result).strip()

    def interactive_run(self, cmd):
        stdin, stdout, stderr = self.ssh_session.exec_command(cmd)
        return stdin, stdout, stderr

    def run_expect_command(self, cmd, expect_end=None, timeout=3, wait_seconds=2):
        """
        执行 shell 命令并获取返回结果
        :param timeout:
        :param wait_seconds:
        :param cmd:
        :param expect_end:
        :return:
        """
        shell = self.ssh_session.invoke_shell()
        last_time = int(time.time())

        if not cmd.endswith('\n'):
            cmd += '\n'

        def receive():
            buff = ''
            if expect_end is None:
                buff = shell.recv(9999)
            else:
                while not buff.endswith(expect_end):
                    resp = shell.recv(9999)
                    buff += force_text(resp)
                    now = int(time.time())

                    if now - last_time > timeout:
                        break

            buff = force_text(buff)
            if self.show_output:
                logger.info(buff)
            return buff
        if self.show_output:
            logger.info(cmd)
        shell.send(cmd)
        time.sleep(wait_seconds)
        return receive()

    def run_nohup(self, cmd, working_dir=None):
        """
        使用重定向输出，正常输出和错误信息都不显示，不会创建 nuhup.out，>/dev/null 2>&1
        :param cmd:
        :param working_dir: 当前的工作目录，如果没有 home 目录，会因为一些原因导致运行失败，比如没有无法创建 nohup.out
        :return:
        """
        cmd = 'nohup %s >/dev/null 2>&1 &\n\n' % cmd
        if working_dir is not None:
            cmd = 'cd {}; {}'.format(working_dir, cmd)

        self.run_expect_command(cmd)

    def check_root(self):
        result = self.run('id')
        return "uid=0" in result, result

    def check_sudo(self):
        """
        TODO 有问题，应该检查用户组
        :return:
        """
        stdin, stdout, stderr = self.ssh_session.exec_command('sudo whoami')
        stdin.write("%s\n" % self.password)
        stdin.write("\n\n\n\n\n\n\n\n")
        stdout.read()
        error_message = stderr.read()[:-1]
        if b"not in the sudoers file" in error_message:
            logger.info('当前用户不在 sudo 组')
            return False
        else:
            logger.info(error_message)
            return True

    def write_ssh_key(self, pub_key):
        logger.info('写 SSH 公钥')
        try:
            id_rsa = open(pub_key, 'r').read().rstrip('\n')
            self.run("mkdir -p ~/.ssh")
            time.sleep(1)
            self.run("chmod 700 ~/.ssh")
            cmd = """echo "{}" >> ~/.ssh/authorized_keys""".format(id_rsa)
            self.run(cmd)
            cmd = "chmod 600 ~/.ssh/authorized_keys"
            self.run(cmd)
            return True
        except Exception as e:
            logger.error(e)
            return False

    def change_password(self, new_password):
        """
        一句话修改密码，但是只有 root 才有权限 echo username:new_password | chpasswd
        :param new_password:
        :return:
        """
        is_root = self.check_root()
        if is_root[0]:
            self.is_root = True
            logger.warning("[+] Root user detected!")
        else:
            self.is_root = False
            logger.warning("[+] Not a root user! (%s)" % is_root[1])

        stdin, stdout, stderr = self.interactive_run('passwd')
        stdin.write("%s\n" % self.password)
        stdin.write("%s\n" % new_password)
        stdin.write("%s\n" % new_password)
        # 通过不停的回车，跳过密码输入错误的重试，避免卡在 stdout.read()
        stdin.write("\n\n\n\n\n\n\n\n\n\n")
        stdout.read()
        error_message = stderr.read()[:-1]
        if b'success' in error_message:
            self.password = new_password
            logger.info('密码修改成功')
            return True
        elif b'unchanged' in error_message:
            logger.info('密码未修改')
            self.password = new_password
            logger.info(error_message)
            return True
        elif b'choose a longer password' in error_message:
            logger.info('密码长度不符合要求')
            return False
        else:
            logger.info(error_message)
            logger.info('密码修改失败')
            return False

    def put(self, local_file, remote_file):
        """
        上传文件
        :param local_file:
        :param remote_file:
        :return:
        """
        sftp = self.get_sftp()
        try:
            sftp.put(local_file, remote_file)
            return True
        except Exception as e:
            logger.error('上传文件失败')
            logger.error('remote: %s, local: %s' % (remote_file, local_file))
            logger.error(e)
            return False

    def get(self, remote_file, local_file):
        """
        下载文件
        :param remote_file:
        :param local_file:
        :return:
        """
        sftp = self.get_sftp()
        try:
            sftp.get(remote_file, local_file)
            return True
        except Exception as e:
            logger.error('下载文件失败')
            logger.error('remote: %s, local: %s' % (remote_file, local_file))
            logger.error(e)
            return False



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

import uuid
from datetime import datetime
from decimal import Decimal
from .. import text_type, PY3, force_text

# https://github.com/esnme/ultrajson
# ujson 速度比较快，但是有些参数没有，与 simplejson 和 json 不完全兼容
# 而且 ujson 似乎已经不再维护了

try:
    import simplejson as json

    simplejson_imported = True
except ImportError:
    import json

    simplejson_imported = False


def json_default(obj):
    """
    对一些数据类型的 json 序列化，
    默认情况下 json 没有对 datetime 和 Decimal 进行序列化
    如果不指定的话，会抛异常
    :param obj:
    :return:
    """
    if isinstance(obj, datetime):
        return obj.strftime("%Y-%m-%d %H:%M:%S")
    elif isinstance(obj, Decimal):
        return float(obj)
    elif isinstance(obj, uuid.UUID):
        return text_type(obj)
    else:
        try:
            return text_type(obj)
        except:
            pass

    raise TypeError("%r is not JSON serializable" % obj)


load = json.load
dump = json.dump


def loads(content, encoding='utf-8', **kwargs):
    if PY3:
        # python3.5，json loads 必须为str类型，如果为bytes类型会报错
        # python3.7，json loads 支持str类型和bytes类型
        content = force_text(content)
    return json.loads(s=content, encoding=encoding, **kwargs)


def dumps(dict_data, ensure_ascii=True, indent=None,
          sort_keys=False, encoding='utf-8', **kwargs):
    """
    返回json数据
    :param encoding:
    :param ensure_ascii:
    :param sort_keys:
    :param indent:
    :param dict_data:
    :return:
    """
    if simplejson_imported:
        return json.dumps(dict_data, default=json_default,
                          ensure_ascii=ensure_ascii, indent=indent,
                          sort_keys=sort_keys, encoding=encoding, **kwargs)
    else:
        return json.dumps(dict_data, default=json_default,
                          ensure_ascii=ensure_ascii, indent=indent,
                          sort_keys=sort_keys, **kwargs)



# -*- coding: utf-8 -*-
# Created by restran on 2017/7/26
from __future__ import unicode_literals, absolute_import

from .base import PY2, PY3, PYPY, string_types, integer_types, \
    class_types, text_type, binary_type, long_type, BytesIO, StringIO
from .encoding import force_text, force_bytes

__author__ = "restran <grestran@gmail.com>"
__version__ = "0.8.8"

__all__ = [
    '__author__', '__version__', 'PY2', 'PY3', 'PYPY',
    'string_types', 'integer_types', 'class_types', 'text_type', 'StringIO',
    'binary_type', 'long_type', 'BytesIO', 'force_text', 'force_bytes',
]



# -*- coding: utf-8 -*-
# Created by restran on 2017/12/11
from __future__ import unicode_literals, absolute_import

import logging
import platform
import time
from collections import deque

from ..base import urlunparse, urlparse, urlencode
from .. import json as json_util
from .. import force_text
from ..http import random_agent

try:
    from tornado.gen import coroutine, is_future, Return
    from tornado.httpclient import AsyncHTTPClient, HTTPRequest, HTTPError
except ImportError:
    raise Exception('tornado is not installed')

logger = logging.getLogger(__name__)

DEFAULT_CONNECT_TIMEOUT = 20
DEFAULT_REQUEST_TIMEOUT = 20
ASYNC_HTTP_MAX_CLIENTS = 100

# pycurl 在 Windows Python 64 位环境下会崩溃
# https://github.com/pycurl/pycurl/issues/395
# libcurl 7.48.0 以后的版本修复了这个bug
if platform.architecture()[0] == '64bit' and platform.system() == 'Windows':
    AsyncHTTPClient.configure(
        'tornado.simple_httpclient.SimpleAsyncHTTPClient')
else:
    try:
        # curl_httpclient is faster than simple_httpclient
        AsyncHTTPClient.configure(
            'tornado.curl_httpclient.CurlAsyncHTTPClient',
            max_clients=ASYNC_HTTP_MAX_CLIENTS)
    except ImportError:
        AsyncHTTPClient.configure(
            'tornado.simple_httpclient.SimpleAsyncHTTPClient')


class AsyncHTTPExecutor(object):
    """
    异步HTTP请求，可以并发访问
    """

    def __init__(self, task_queue, on_request,
                 on_success, on_error, on_queue_empty=None,
                 proxy_list=None,
                 max_workers=20, timeout=5, verbose=False):
        self.fn_on_queue_empty = on_queue_empty
        self.fn_on_request = on_request
        self.fn_on_success = on_success
        self.fn_on_error = on_error
        self.task_queue = deque()
        self.task_queue.extend(task_queue)
        self.timeout = timeout
        self.max_workers = max_workers
        self.count = 0
        self.verbose = verbose
        self.start_time = None
        self.last_time = None
        self.proxy_list = proxy_list
        self.proxy_index = 0

    def get_proxy(self):
        """
        获取下一个可用的代理
        :return: proxy_host, proxy_port, proxy_username, proxy_password
        """
        if self.proxy_list is None or len(self.proxy_list) <= 0:
            return None, None, None, None
        self.proxy_index += 1
        self.proxy_index = self.proxy_index % len(self.proxy_list)
        item = self.proxy_list[self.proxy_index]
        if len(item) == 2:
            return item[0], item[1], None, None
        else:
            return item[0], item[1], item[2], item[3]

    def get_next_task(self):
        try:
            item = self.task_queue.popleft()
        except IndexError:
            if self.fn_on_queue_empty is None:
                return None
            else:
                item = self.fn_on_queue_empty(self.task_queue)
                if isinstance(item, list) and len(item) > 0:
                    self.task_queue.extend(item)
                    item = self.task_queue.popleft()
        return item

    @coroutine
    def do_request(self, item):
        self.count += 1
        current_time = time.time()
        # 每隔10秒输出一次
        if current_time - self.last_time > 10:
            self.last_time = current_time
            speed = self.count / (current_time - self.start_time)
            past_time = current_time - self.start_time
            logger.info('items, speed, time: %s, %.1f/s, %.1fs' % (self.count, speed, past_time))

        base_headers = {
            'User-Agent': random_agent()
        }

        url, method, headers, body, extra_params = self.fn_on_request(item)
        if url is None:
            return

        if headers is None:
            headers = {}

        base_headers.update(headers)
        headers = base_headers

        url = force_text(url)
        body = '' if method == 'POST' else None
        proxy_host, proxy_port, proxy_username, proxy_password = self.get_proxy()
        if proxy_port is not None:
            proxy_port = int(proxy_port)

        params_dict = {
            'decompress_response': True,
            'validate_cert': False,
            'proxy_host': proxy_host,
            'proxy_port': proxy_port,
            'proxy_username': proxy_username,
            'proxy_password': proxy_password,
            'connect_timeout': self.timeout,
            'request_timeout': self.timeout,
            'follow_redirects': False
        }
        # 允许再传其他参数
        if isinstance(extra_params, dict):
            params_dict.update(extra_params)

        try:
            response = yield AsyncHTTPClient().fetch(
                HTTPRequest(url=url,
                            method=method,
                            headers=headers,
                            body=body,
                            **params_dict))
            try:
                self.fn_on_success(url, item, method, response, self.task_queue)
            except Exception as e:
                logger.error(e)
        except HTTPError as e:
            if hasattr(e, 'response') and e.response:
                try:
                    self.fn_on_success(url, item, method, e.response, self.task_queue)
                except Exception as ex:
                    logger.error(ex)
            else:
                if self.verbose:
                    logger.error(e)
                    logger.error('%s, %s' % (method, item))
                try:
                    self.fn_on_error(url, item, method, e, self.task_queue)
                except Exception as e:
                    logger.error(e)
        except Exception as e:
            if self.verbose:
                logger.error(e)
                logger.error('%s, %s' % (method, item))

            try:
                self.fn_on_error(url, item, method, e, self.task_queue)
            except Exception as e:
                logger.error(e)

    @coroutine
    def fetch_url(self, i):
        item = self.get_next_task()
        while item is not None:
            yield self.do_request(item)
            item = self.get_next_task()

    @coroutine
    def run(self, *args, **kwargs):
        logger.info('executor start')
        self.start_time = time.time()
        self.last_time = self.start_time
        # Start workers, then wait for the work queue to be empty.
        # 会卡在这里，等待所有的 worker 都结束
        yield [self.fetch_url(t) for t in range(self.max_workers)]
        end_time = time.time()
        logger.info('total count: %s' % self.count)
        cost_time = end_time - self.start_time
        if cost_time > 0:
            speed = self.count / cost_time
        else:
            speed = 1

        logger.info('executor done, %.3f, %.1f/s' % (cost_time, speed))


@coroutine
def async_request(method='GET', url=None, params=None,
                  headers=None, data=None, json=None,
                  on_response=None, on_error=None,
                  connect_timeout=DEFAULT_CONNECT_TIMEOUT,
                  request_timeout=DEFAULT_REQUEST_TIMEOUT,
                  follow_redirects=False,
                  proxy_host=None, proxy_port=None,
                  proxy_username=None, proxy_password=None):
    try:
        if url is None:
            return

        method = method.upper()

        base_headers = {
            'User-Agent': random_agent(),
        }

        if params is not None:
            url_parsed = urlparse(url)
            query = urlencode(params, doseq=True)
            if url_parsed.query != '':
                query = '%s&%s' % (query, url_parsed.query)

            url = urlunparse((url_parsed.scheme, url_parsed.netloc,
                              url_parsed.path, url_parsed.params,
                              query, url_parsed.fragment))

        if method == 'GET':
            body = None
        else:
            if json is not None:
                body = json_util.dumps(json)
                base_headers['Content-Type'] = 'application/json;charset=utf-8'
            elif isinstance(data, dict):
                body = urlencode(data, doseq=True)
                base_headers['Content-Type'] = 'application/x-www-form-urlencoded'
            elif isinstance(data, list):
                body = force_text(data)
            else:
                body = data

        if isinstance(headers, dict):
            base_headers.update(headers)

        headers = base_headers
        response = yield AsyncHTTPClient().fetch(
            HTTPRequest(url=url,
                        headers=headers,
                        method=method,
                        body=body,
                        validate_cert=False,
                        decompress_response=True,
                        connect_timeout=connect_timeout,
                        request_timeout=request_timeout,
                        follow_redirects=follow_redirects,
                        proxy_host=proxy_host,
                        proxy_port=proxy_port,
                        proxy_username=proxy_username,
                        proxy_password=proxy_password))

        if on_response is not None:
            ret = on_response(response)
            if is_future(ret):
                yield ret

        raise Return(response)
    except Return as e:
        # 上面有抛出 Return 的异常，这里捕获到以后，要重新抛出
        # 否则返回的实际上就不是 response
        raise e
    except HTTPError as e:
        if hasattr(e, 'response') and e.response:
            if on_response is not None:
                ret = on_response(e.response)
                if is_future(ret):
                    yield ret

            raise Return(e.response)
        else:
            if on_error is not None:
                ret = on_error(e)
                if is_future(ret):
                    yield ret
            raise Return(None)
    except Exception as e:
        if on_error is not None:
            ret = on_error(e)
            if is_future(ret):
                yield ret

        raise Return(None)



# -*- coding: utf-8 -*-
# Created by restran on 2016/11/1
from __future__ import unicode_literals, absolute_import

import time
from datetime import datetime

"""
date string, datetime, time and timestamp converter
"""


def str2datetime(date_str, format='%Y-%m-%d %H:%M:%S'):
    try:
        return datetime.strptime(date_str, format)
    except:
        return None


def str2time(date_str, format='%Y-%m-%d %H:%M:%S'):
    try:
        return time.strptime(date_str, format)
    except:
        return None


def str2timestamp(date_str, format='%Y-%m-%d %H:%M:%S', millisecond=False):
    try:
        ts = time.mktime(time.strptime(date_str, format))
        if millisecond:
            ts = ts * 1000
        return int(ts)
    except:
        return None


def datetime2str(dt, format='%Y-%m-%d %H:%M:%S'):
    try:
        return dt.strftime(format)
    except:
        return None


def datetime2time(dt):
    try:
        return dt.timetuple()
    except:
        return None


def datetime2timestamp(dt, millisecond=False):
    try:
        ts = time.mktime(dt.timetuple())
        if millisecond:
            ts = ts * 1000
        return int(ts)
    except:
        return None


def time2str(time_tuple, format='%Y-%m-%d %H:%M:%S'):
    try:
        return time.strftime(format, time_tuple)
    except:
        return None


def time2datetime(time_tuple):
    try:
        return datetime(*time_tuple[0:6])
    except:
        return None


def time2timestamp(time_tuple, millisecond=False):
    try:
        ts = time.mktime(time_tuple)
        if millisecond:
            ts = ts * 1000
        return int(ts)
    except:
        return None


def timestamp2datetime(ts, millisecond=False):
    try:
        if millisecond:
            ts = int(ts / 1000.0)
        return datetime.fromtimestamp(ts)
    except:
        return None


def timestamp2time(ts, millisecond=False):
    try:
        if millisecond:
            ts = int(ts / 1000.0)
        return time.localtime(ts)
    except:
        return None


def timestamp2str(ts, format='%Y-%m-%d %H:%M:%S', millisecond=False):
    try:
        if millisecond:
            ts = int(ts / 1000.0)
        return datetime.fromtimestamp(ts).strftime(format)
    except:
        return None



# -*- coding: utf-8 -*-
# created by restran on 2016/07/02
from __future__ import unicode_literals, absolute_import, print_function

import logging
import os
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
from email.header import make_header

from .. import BytesIO, PY2

logger = logging.getLogger(__name__)


class EmailHandler(object):
    def __init__(self, mail_from, password, smtp_server, smtp_port=25):
        """
        :param mail_from: 发件人
        :param password: 发件人密码
        :param smtp_server: SMTP服务器地址
        :param smtp_port: SMTP服务器端口，SSL 方式是 465
        :return:
        """

        self.mail_from = mail_from
        self.password = password
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port

    def send_mail_ssl(self, mail_to_list, subject, content, file_name_list):
        self.do_send_mail(True, mail_to_list, subject, content, file_name_list)

    def send_mail(self, mail_to_list, subject, content, file_name_list):
        self.do_send_mail(False, mail_to_list, subject, content, file_name_list)

    def do_send_mail(self, is_ssl, mail_to_list, subject, content, file_name_list):
        """
        发送邮件
        :param is_ssl: 使用SSL的方式发生
        :param mail_to_list: 收件人列表
        :param subject: 邮件主题
        :param content: 邮件正文
        :param file_name_list: 附近的文件路径列表
        :return:
        """
        if is_ssl:
            smtp = smtplib.SMTP_SSL(self.smtp_server, self.smtp_port)
        else:
            smtp = smtplib.SMTP(self.smtp_server, self.smtp_port)
        smtp.ehlo(name='foxmail')
        # 调用login时，如果没有调用过 echlo 会自动调用该方法，但是默认使用的name为计算机名
        # 如果计算机名有中文，就会返回503方法未实现的异常
        smtp.login(self.mail_from, self.password)
        msg = MIMEMultipart()
        msg['From'] = self.mail_from
        msg['To'] = ', '.join(mail_to_list)
        msg['Date'] = formatdate(localtime=True)
        msg['Subject'] = subject
        # 如果 content 是 html，则需要设置 _subtype='html'
        # 默认情况下 _subtype='plain'，即纯文本
        msg.attach(MIMEText(content, _charset='UTF-8'))
        for fn in file_name_list:
            part = MIMEText(open(fn, 'rb').read(), 'base64', 'UTF-8')
            part["Content-Type"] = 'application/octet-stream'
            basename = os.path.basename(fn)
            # 解决一些邮箱在手机客户端上，附件文件名丢失的问题
            part["Content-Type"] = 'application/octet-stream;name="%s"'% make_header([(basename,'UTF-8')]).encode('UTF-8')
            part["Content-Disposition"] = 'attachment;filename="%s"' % make_header([(basename, 'UTF-8')]).encode('UTF-8')
            msg.attach(part)
        smtp.sendmail(self.mail_from, mail_to_list, msg.as_string())
        smtp.close()


class PostfixEmailHandler(object):
    def __init__(self, mail_from, smtp_server='localhost', smtp_port=25):
        self.mail_from = mail_from
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port

    def send_email(self, mail_to, subject, content, content_type='plain', files=None):
        """
        :param content_type: 如果 text 是html，则需要设置 _subtype='html'
        :param mail_to:
        :param subject:
        :param content:
        :param files: (f_name, f_data)
        :return:
        """
        assert type(mail_to) == list
        server = self.smtp_server
        if files is None:
            files = []

        msg = MIMEMultipart()
        msg['From'] = self.mail_from
        msg['To'] = ', '.join(mail_to)
        msg['Date'] = formatdate(localtime=True)
        msg['Subject'] = subject
        # 如果 text 是html，则需要设置 _subtype='html'
        # 默认情况下 _subtype='plain'，即纯文本
        msg.attach(MIMEText(content, _subtype=content_type, _charset='utf-8'))

        for fn, fd in files:
            part = MIMEText(fd, 'base64', 'utf-8')
            part["Content-Type"] = 'application/octet-stream'
            basename = fn
            if PY2:
                basename = basename.encode('gb2312')
            # 文件名使用 gb2312 编码，否则会没有附件
            part.add_header('Content-Disposition', 'attachment', filename=('gb2312', '', basename))
            msg.attach(part)
        smtp = smtplib.SMTP(server, port=self.smtp_port)
        smtp.sendmail(self.mail_from, mail_to, msg.as_string())
        smtp.close()


def main():
    handler = EmailHandler('mail_from@example.com', 'password', 'smtp.example.com', 465)
    mail_to_list = ['example@test.com']
    subject = 'Python 发送邮件测试'
    content = '这是用 Python 自动发送的邮件，请勿回复'
    # 附件存放在当前文件夹
    file_name_list = ['test.rar']
    handler.send_mail_ssl(mail_to_list, subject, content, file_name_list)
    print('邮件发送成功')

    mail_to_list = ['mail_from@example.com']
    subject = 'Python 发送邮件测试'
    content = '这是用 Python 自动发送的邮件，请勿回复'
    bio = BytesIO()
    bio.write(b'123')
    bio.getvalue()
    files = [
        ('test中文3.txt', bio.getvalue()),
    ]
    handler = PostfixEmailHandler('mail_from@example.com')
    handler.send_email(mail_to_list, subject, content, files=files)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# created by restran on 2018/01/22
from __future__ import unicode_literals, absolute_import

import logging
import traceback
import math

try:
    from django.core.paginator import Paginator, EmptyPage
except ImportError:
    raise Exception('django is not installed')

from ..base import iteritems

MAX_PAGE_SIZE = 100

DEFAULT_PAGE_SIZE = 20

logger = logging.getLogger(__name__)


def object_set_dict_data(model_class, dict_data):
    obj = model_class()
    for k, v in iteritems(dict_data):
        setattr(obj, k, v)

    return obj


def add_fields_2_json(obj, json_dict, fields):
    for t in fields:
        if hasattr(obj, t):
            json_dict[t] = getattr(obj, t)


def set_dict_none_default(dict_item, default_value):
    """
    对字典中为None的值，重新设置默认值
    :param dict_item:
    :param default_value:
    :return:
    """
    for (k, v) in iteritems(dict_item):
        if v is None:
            dict_item[k] = default_value


def auto_model_name_recognize(model_name):
    """
    自动将 site-user 识别成 SiteUser
    :param model_name:
    :return:
    """
    name_list = model_name.split('-')
    return ''.join(['%s%s' % (name[0].upper(), name[1:]) for name in name_list])


def model_get_entry(model_class, entry_id=None, filter_dict=None, select_related_fields=None):
    """
    """
    if filter_dict is None:
        filter_dict = {}

    if entry_id is not None:
        filter_dict['id'] = entry_id

    try:
        if select_related_fields is None:
            return model_class.objects.get(**filter_dict)
        else:
            fields = [t.attname for t in model_class._meta.fields]
            fields.extend(select_related_fields)
            obj = model_class.objects.values(*fields).get(**filter_dict)
            return object_set_dict_data(model_class, obj)
    except model_class.DoesNotExist:
        return None
    except Exception as e:
        logger.error(traceback.format_exc())
        logger.error(e)
        return None


def model_delete_entry(model_class, entry_id):
    model_class.objects.filter(id=entry_id).delete()


def model_total_count(model_class, filter_dict=None, q_filter=None):
    if filter_dict is None:
        filter_dict = {}

    if q_filter is not None:
        filter_list = [q_filter]
    else:
        filter_list = []

    return model_class.objects.filter(
        *filter_list, **filter_dict).count()


def model_to_select_list(model_class, filter_dict=None, q_filter=None):
    """
    只选择 id 和 name，用来做列表选择
    :param model_class:
    :param filter_dict:
    :param q_filter:
    :return:
    """
    if filter_dict is None:
        filter_dict = {}

    if q_filter is not None:
        filter_list = [q_filter]
    else:
        filter_list = []

    objects = model_class.objects.filter(
        *filter_list, **filter_dict).values('id', 'name')

    return list(objects)


def model_to_list(model_class, filter_dict=None, order_by_list=None,
                  select_related_fields=None, q_filter=None,
                  values=None,
                  to_json_method='to_json'):
    """
    不分页
    :param values:
    :param to_json_method:
    :param model_class:
    :param filter_dict:
    :param order_by_list:
    :param select_related_fields:
    :param q_filter:
    :return:
    """
    return model_to_page_list(model_class, page_num=None,
                              filter_dict=filter_dict, order_by_list=order_by_list,
                              select_related_fields=select_related_fields,
                              q_filter=q_filter, values=values,
                              to_json_method=to_json_method)


def model_to_page_list(model_class, page_num,
                       page_size=DEFAULT_PAGE_SIZE,
                       filter_dict=None, order_by_list=None,
                       select_related_fields=None, q_filter=None,
                       values=None, to_json_method='to_json',
                       max_page_size=MAX_PAGE_SIZE):
    """
    :param max_page_size:
    :param model_class:
    :param page_num:
    :param page_size:
    :param filter_dict:
    :param order_by_list:
    :param select_related_fields:
    :param q_filter: Q(uuid__contains=keyword) | Q(memo__contains=keyword)
    :param values:
    :param to_json_method:
    :return:
    """
    if order_by_list is None:
        order_by_list = ['-id']

    if filter_dict is None:
        filter_dict = {}

    if q_filter is not None:
        filter_list = [q_filter]
    else:
        filter_list = []

    if select_related_fields is None:
        if values is None:
            objects = model_class.objects.filter(
                *filter_list, **filter_dict).order_by(*order_by_list)
        else:
            objects = model_class.objects.filter(
                *filter_list, **filter_dict).values(*values).order_by(*order_by_list)
    else:
        if values is None:
            fields = [t.attname for t in model_class._meta.fields]
        else:
            fields = values
        fields.extend(select_related_fields)
        objects = model_class.objects.filter(
            *filter_list, **filter_dict).values(*fields).order_by(*order_by_list)

    if page_num is not None:
        if page_size > max_page_size:
            page_size = max_page_size

        paginator = Paginator(objects, page_size)
        try:
            obj_list = paginator.page(page_num)
        except EmptyPage as e:
            obj_list = []
    else:
        obj_list = list(objects)

    if select_related_fields is not None or values is not None:
        obj_list = [object_set_dict_data(model_class, t) for t in obj_list]

    if to_json_method is None:
        return obj_list
    else:
        return [getattr(t, to_json_method)() for t in obj_list]


def bulk_delete(model, delete_id_list, id_name='id', batch_size=200):
    """
    批量删除
    :param model:
    :param delete_id_list:
    :param id_name: id的名称，一般是id
    :param batch_size:
    :return:
    """
    total = len(delete_id_list)
    times = math.ceil(total * 1.0 / batch_size)
    for i in range(times):
        data_list = delete_id_list[i * batch_size:(i + 1) * batch_size]
        if len(data_list) > 0:
            id_list = [t for t in data_list]
            filter_params = {
                '{}__in'.format(id_name): id_list
            }
            model.objects.filter(**filter_params).delete()



# -*- coding: utf-8 -*-
# Created by restran on 2017/9/15
from __future__ import unicode_literals, absolute_import

import time
from functools import wraps


# 装饰器的本质，是把函数作为参数传给装饰器函数
# 装饰器实际上是闭包
def log(func):
    """
    打印函数运行日志的装饰器
    :param func:
    :return:
    """

    # wraps 这个装饰器会将 func 的 doc 和 __name__ 复制过来
    # 否则在 wrapper 中调用 func.__name__ 输出的就是 wrapper
    @wraps(func)
    def wrapper(*args, **kwargs):
        print('before call %s' % func.__name__)
        ret = func(*args, **kwargs)
        print('after call %s' % func.__name__)
        return ret

    return wrapper


def log_with_message(message):
    """
    打印函数运行日志的装饰器，可以再给装饰器传参数
    :param message:
    :return:
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            print('decorator log_with_message is running, %s' % message)
            ret = func(*args, **kwargs)
            return ret

        return wrapper

    return decorator


def time_elapsed(func):
    """
    记录函数运行耗时的生成器
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        timestamp = time.time() * 1000
        ret = func(*args, **kwargs)
        now_ts = time.time() * 1000
        elapsed = now_ts - timestamp
        print('%s costs time: %.2fms' % (func.__name__, elapsed))
        return ret

    return wrapper


# -*- coding: utf-8 -*-
# Created by restran on 2018/9/9
from __future__ import unicode_literals, absolute_import
import sys
from .terminal_size import get_terminal_size

TERMINAL_SIZE = get_terminal_size()


def terminal_pos(y, x):
    return '\x1b[%d;%dH' % (y, x)


def print_on_terminal_bottom(text):
    print('%s%s' % (terminal_pos(TERMINAL_SIZE[0], TERMINAL_SIZE[1]), text), end='')


def print_on_terminal_fix(text):
    sys.stdout.write("%s\r" % text)
    sys.stdout.flush()


class ColorConsole(object):
    GREEN = "\033[92m"
    BLUE = "\033[94m"
    BOLD = "\033[1m"
    YELLOW = "\033[93m"
    RED = "\033[91m"
    END = "\033[0m"

    @classmethod
    def green(cls, message):
        return '%s%s%s' % (cls.GREEN, message, cls.END)

    @classmethod
    def blue(cls, message):
        return '%s%s%s' % (cls.BLUE, message, cls.END)

    @classmethod
    def red(cls, message):
        return '%s%s%s' % (cls.RED, message, cls.END)

    @classmethod
    def yellow(cls, message):
        return '%s%s%s' % (cls.YELLOW, message, cls.END)

    @classmethod
    def bold(cls, message):
        return '%s%s%s' % (cls.BOLD, message, cls.END)



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

import logging
import logging.config

FORMAT_VERBOSE = "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s"
# Python的logging使用的是time的datefmt，并没有提供毫秒的datefmt
# http://blog.csdn.net/arthur503/article/details/49359241
DATE_FMT_VERBOSE = "%Y-%m-%d %H:%M:%S"

FORMAT_SIMPLE = '%(levelname)s %(message)s'
DATE_FMT_SIMPLE = "%H:%M:%S"

DEBUG = 'DEBUG'
INFO = 'INFO'
WARNING = 'WARNING'
ERROR = 'ERROR'
CRITICAL = 'CRITICAL'


def getLogger(name):
    """
    用于替代 logging.getLogger
    :param name:
    :return:
    """
    return logging.getLogger(name)


def Logger(name):
    """
    用于替代 logging.getLogger
    :param name:
    :return:
    """
    return logging.getLogger(name)


class BaseHandler(object):
    def __init__(self, level=DEBUG, format=None, datefmt=None):
        self.level = level
        self.format = format
        self.datefmt = datefmt

        if self.datefmt is None:
            self.datefmt = DATE_FMT_VERBOSE

        if self.format is None:
            self.format = FORMAT_VERBOSE

        self.handler_class = 'logging.handlers.StreamHandler'

    def get_formatter_name(self):
        return self.__class__.__name__

    def get_formatter(self):
        formatter = {
            self.get_formatter_name(): {
                'format': self.format,
                'datefmt': self.datefmt,
            }
        }

        return formatter

    def get_handler(self):
        handler = {
            self.get_formatter_name(): {
                'level': self.level,
                'class': self.handler_class,
                'formatter': self.get_formatter_name()
            }
        }

        return handler


class StreamHandler(BaseHandler):
    def __init__(self, level=DEBUG, format=None, datefmt=None):
        super(StreamHandler, self).__init__(level, format, datefmt)
        self.handler_class = 'logging.StreamHandler'


class ColorStreamHandler(BaseHandler):
    def __init__(self, level=DEBUG, format=None, datefmt=None, log_colors=None):
        super(ColorStreamHandler, self).__init__(level, format, datefmt)
        self.format = '%(log_color)s' + self.format
        self.handler_class = 'colorlog.StreamHandler'
        try:
            import colorlog
        except:
            raise Exception('colorlog not installed')

        if log_colors is None:
            log_colors = {
                'DEBUG': 'white',
                'INFO': 'green',
                'WARNING': 'yellow',
                'ERROR': 'red',
                # 'CRITICAL': 'red',
                'CRITICAL': 'red, bg_white',
            }
        self.log_colors = log_colors

    def get_formatter(self):
        formatter = {
            self.get_formatter_name(): {
                '()': 'colorlog.ColoredFormatter',
                'format': self.format,
                'datefmt': self.datefmt,
                'log_colors': self.log_colors
            }
        }

        return formatter


class FileHandler(BaseHandler):
    def __init__(self, filename='log.txt', level=DEBUG, format=None, datefmt=None, encoding='utf8'):
        super(FileHandler, self).__init__(level, format, datefmt)
        self.filename = filename
        self.handler_class = 'logging.FileHandler'
        self.encoding = encoding

    def get_handler(self):
        handler = super(FileHandler, self).get_handler()
        new_params = {
            'filename': self.filename,
            'encoding': self.encoding
        }

        handler[self.get_formatter_name()].update(new_params)
        return handler


class RotatingFileHandler(BaseHandler):
    def __init__(self, filename='log.txt', max_bytes=1024 * 1024 * 10,
                 backup_count=10, delay=True, level=DEBUG,
                 format=None, datefmt=None, encoding='utf8'):
        super(RotatingFileHandler, self).__init__(level, format, datefmt)
        self.filename = filename
        self.handler_class = 'logging.handlers.RotatingFileHandler'
        self.max_bytes = max_bytes
        self.backup_count = backup_count
        self.delay = delay
        self.encoding = encoding

    def get_handler(self):
        handler = super(RotatingFileHandler, self).get_handler()
        new_params = {
            'filename': self.filename,
            'delay': self.delay,
            'maxBytes': self.max_bytes,
            'backupCount': self.backup_count,
            'encoding': self.encoding
        }

        handler[self.get_formatter_name()].update(new_params)
        return handler


class TimedRotatingFileHandler(BaseHandler):
    def __init__(self, filename='log.txt', when='D', interval=1,
                 backup_count=10, delay=True, level=DEBUG,
                 format=None, datefmt=None, encoding='utf8'):
        """
        when 可以使用这些参数
        'S' Seconds
        'M' Minutes
        'H' Hours
        'D' Days
        'W0'-'W6' Weekday (0=Monday)
        'midnight' Roll over at midnight
        :param filename:
        :param when:
        :param interval:
        :param backup_count:
        :param delay:
        :param level:
        :param format:
        :param datefmt:
        :param encoding:
        """
        super(TimedRotatingFileHandler, self).__init__(level, format, datefmt)
        self.filename = filename
        self.handler_class = 'logging.handlers.TimedRotatingFileHandler'
        self.when = when
        self.interval = interval
        self.delay = delay
        self.backup_count = backup_count
        self.encoding = encoding

    def get_handler(self):
        handler = super(TimedRotatingFileHandler, self).get_handler()
        new_params = {
            'filename': self.filename,
            'delay': self.delay,
            'when': self.when,
            'interval': self.interval,
            'backupCount': self.backup_count,
            'encoding': self.encoding
        }

        handler[self.get_formatter_name()].update(new_params)
        return handler


def init_log(*handlers, **kwargs):
    """
    :param handlers:
    :return:
    """
    disable_existing_loggers = kwargs.get('disable_existing_loggers', False)

    handlers_config = [t.get_handler() for t in handlers]
    new_handlers_config = {}
    for t in handlers_config:
        new_handlers_config.update(t)

    formatter_config = [t.get_formatter() for t in handlers]
    new_formatter_config = {}
    for t in formatter_config:
        new_formatter_config.update(t)

    handler_name_list = [t.get_formatter_name() for t in handlers]
    dict_config = {
        'version': 1,
        'disable_existing_loggers': disable_existing_loggers,
        'formatters': new_formatter_config,
        'handlers': new_handlers_config,
        'loggers': {
            '': {
                'handlers': handler_name_list,
                'level': 'DEBUG',
            }
        }
    }

    logging.config.dictConfig(dict_config)



# -*- coding: utf-8 -*-
# Created by restran on 2018/3/22
from __future__ import unicode_literals, absolute_import

from mountains.utils import string_utils
import unittest
import random
import string


class UtilsTest(unittest.TestCase):
    def test_line_break(self):
        s = ''.join([random.choice(string.ascii_letters + string.digits) for _ in range(3000)])
        break_s = string_utils.line_break(s)
        self.assertEqual(s, break_s.replace('\n', ''))

        break_s = string_utils.line_break(s, 10)
        self.assertEqual(s, break_s.replace('\n', ''))

        break_s = string_utils.line_break(s, 100)
        self.assertEqual(s, break_s.replace('\n', ''))

        break_s = string_utils.line_break(s, 3001)
        self.assertEqual(s, break_s.replace('\n', ''))

    def test_fixed_length_split(self):
        s = 'aaaabbbbccccdddd'
        r = string_utils.fixed_length_split(s, 4)
        self.assertEqual(r, ['aaaa', 'bbbb', 'cccc', 'dddd'])
        r = string_utils.fixed_length_split(s, 6)
        self.assertEqual(r, ['aaaabb', 'bbcccc', 'dddd'])
        r = string_utils.fixed_length_split(s, 16)
        self.assertEqual(r, ['aaaabbbbccccdddd'])

    def test_is_empty(self):
        self.assertEqual(string_utils.is_empty(''), True)
        self.assertEqual(string_utils.is_empty(None), True)
        self.assertEqual(string_utils.is_empty('aaa'), False)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# created by restran on 2018/01/22
from __future__ import unicode_literals, absolute_import


def str2int(number_str, default_value=None):
    if number_str is None or number_str == '':
        return default_value
    try:
        return int(number_str)
    except Exception as e:
        return default_value



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import
import re
from ..base import text_type, string_types
import string
from base64 import b64decode, b32decode

_UTF8_TYPES = (bytes, type(None))

_TO_UNICODE_TYPES = (text_type, type(None))


def utf8(value, encoding=None):
    """Converts a string argument to a byte string.
    """
    if isinstance(value, _UTF8_TYPES):
        return value
    if not isinstance(value, text_type):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )

    if encoding is not None:
        return value.encode(encoding)

    try:
        return value.encode('utf-8')
    except:
        return value.encode('gbk')


def to_unicode(value, encoding=None):
    """Converts a string argument to a unicode string.
    """
    if isinstance(value, text_type):
        return value
    elif isinstance(value, type(None)):
        return text_type(value)

    if not isinstance(value, bytes):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )

    if encoding is not None:
        return value.decode(encoding)
    else:
        try:
            value = value.decode('utf-8')
        except:
            try:
                value = value.decode('gbk')
            except:
                pass

        return value


def force_text(s, encoding=None):
    """
    强制转成 Unicode
    """

    return to_unicode(s, encoding)


def force_bytes(s, encoding=None):
    """
    强制转成 bytes
    """

    return utf8(s, encoding)


def is_base64(s, is_printable=True):
    if len(s) % 4 != 0:
        return False

    b64rex = re.compile('^[A-Za-z0-9+/]+[=]{0,2}$', re.MULTILINE)
    if not b64rex.match(s):
        return False

    if is_printable:
        try:
            a = b64decode(s.encode()).decode()
            for c in a:
                if c not in string.printable:
                    return False
        except:
            return False
    return True


def is_base32(s, is_printable=True):
    if len(s) % 8 != 0:
        return False

    rex = re.compile('^[A-Z2-7]+[=]{0,7}$', re.MULTILINE)
    if not rex.match(s):
        return False

    if is_printable:
        try:
            a = b32decode(s.encode()).decode()
            for c in a:
                if c not in string.printable:
                    return False
        except:
            return False
    return True


def main():
    pass


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

from mountains import logging
from mountains.logging import StreamHandler, FileHandler, RotatingFileHandler, TimedRotatingFileHandler
import unittest


class UtilsTest(unittest.TestCase):
    def test_1(self):
        logging.init_log(StreamHandler(format=logging.FORMAT_SIMPLE),
                         FileHandler(format=logging.FORMAT_VERBOSE, level=logging.DEBUG))
        logger = logging.getLogger(__name__)
        print('123')
        logger.debug('hello')

    def test_2(self):
        logging.init_log(StreamHandler(format=logging.FORMAT_SIMPLE),
                         RotatingFileHandler(format=logging.FORMAT_VERBOSE, level=logging.DEBUG))
        logger = logging.getLogger(__name__)

        logger.debug('hello')

    def test_3(self):
        logging.init_log(StreamHandler(format=logging.FORMAT_SIMPLE),
                         TimedRotatingFileHandler(format=logging.FORMAT_VERBOSE, level=logging.DEBUG))
        logger = logging.getLogger(__name__)

        logger.debug('hello')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# created by restran on 2019/04/08
from __future__ import unicode_literals, absolute_import

"""
Original from https://github.com/googollee/eviltransform
地球坐标（WGS-84）
火星坐标（GCJ－2）
GCJ-02坐标用在谷歌地图，高德地图、腾讯地图等中国地图服务。
百度地图要在GCJ-02基础上再加转换
"""

import math

__all__ = ['wgs2gcj', 'gcj2wgs', 'gcj2wgs_exact',
           'distance', 'gcj2bd', 'bd2gcj', 'wgs2bd', 'bd2wgs']

earth_r = 6378137.0


def out_of_china(lat, lng):
    return not (72.004 <= lng <= 137.8347 and 0.8293 <= lat <= 55.8271)


def transform(x, y):
    xy = x * y
    abs_x = math.sqrt(abs(x))
    x_pi = x * math.pi
    y_pi = y * math.pi
    d = 20.0 * math.sin(6.0 * x_pi) + 20.0 * math.sin(2.0 * x_pi)

    lat = d
    lng = d

    lat += 20.0 * math.sin(y_pi) + 40.0 * math.sin(y_pi / 3.0)
    lng += 20.0 * math.sin(x_pi) + 40.0 * math.sin(x_pi / 3.0)

    lat += 160.0 * math.sin(y_pi / 12.0) + 320 * math.sin(y_pi / 30.0)
    lng += 150.0 * math.sin(x_pi / 12.0) + 300.0 * math.sin(x_pi / 30.0)

    lat *= 2.0 / 3.0
    lng *= 2.0 / 3.0

    lat += -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * y + 0.1 * xy + 0.2 * abs_x
    lng += 300.0 + x + 2.0 * y + 0.1 * x * x + 0.1 * xy + 0.1 * abs_x

    return lat, lng


def delta(lat, lng):
    ee = 0.00669342162296594323
    d_lat, d_lng = transform(lng - 105.0, lat - 35.0)
    rad_lat = lat / 180.0 * math.pi
    magic = math.sin(rad_lat)
    magic = 1 - ee * magic * magic
    sqrt_magic = math.sqrt(magic)
    d_lat = (d_lat * 180.0) / ((earth_r * (1 - ee)) / (magic * sqrt_magic) * math.pi)
    d_lng = (d_lng * 180.0) / (earth_r / sqrt_magic * math.cos(rad_lat) * math.pi)
    return d_lat, d_lng


def wgs2gcj(wgs_lat, wgs_lng):
    if out_of_china(wgs_lat, wgs_lng):
        return wgs_lat, wgs_lng
    else:
        d_lat, d_lng = delta(wgs_lat, wgs_lng)
        return wgs_lat + d_lat, wgs_lng + d_lng


def gcj2wgs(gcj_lat, gcj_lng):
    if out_of_china(gcj_lat, gcj_lng):
        return gcj_lat, gcj_lng
    else:
        d_lat, d_lng = delta(gcj_lat, gcj_lng)
        return gcj_lat - d_lat, gcj_lng - d_lng


def gcj2wgs_exact(gcj_lat, gcj_lng):
    init_delta = 0.01
    threshold = 0.000001
    d_lat = d_lng = init_delta
    m_lat = gcj_lat - d_lat
    m_lng = gcj_lng - d_lng
    p_lat = gcj_lat + d_lat
    p_lng = gcj_lng + d_lng
    for i in range(30):
        wgs_lat = (m_lat + p_lat) / 2
        wgs_lng = (m_lng + p_lng) / 2
        tmp_lat, tmp_lng = wgs2gcj(wgs_lat, wgs_lng)
        d_lat = tmp_lat - gcj_lat
        d_lng = tmp_lng - gcj_lng
        if abs(d_lat) < threshold and abs(d_lng) < threshold:
            return wgs_lat, wgs_lng
        if d_lat > 0:
            p_lat = wgs_lat
        else:
            m_lat = wgs_lat
        if d_lng > 0:
            p_lng = wgs_lng
        else:
            m_lng = wgs_lng
    return wgs_lat, wgs_lng


def distance(lat_a, lng_a, lat_b, lng_b):
    """
    计算两个经纬度之间的距离
    :param lat_a:
    :param lng_a:
    :param lat_b:
    :param lng_b:
    :return:
    """
    pi180 = math.pi / 180
    arc_lat_a = lat_a * pi180
    arc_lat_b = lat_b * pi180
    x = (math.cos(arc_lat_a) * math.cos(arc_lat_b) *
         math.cos((lng_a - lng_b) * pi180))
    y = math.sin(arc_lat_a) * math.sin(arc_lat_b)
    s = x + y
    if s > 1:
        s = 1
    if s < -1:
        s = -1
    alpha = math.acos(s)
    d = alpha * earth_r
    return d


def gcj2bd(gcj_lat, gcj_lng):
    """
    GCJ－2转百度
    :param gcj_lat:
    :param gcj_lng:
    :return:
    """
    if out_of_china(gcj_lat, gcj_lng):
        return gcj_lat, gcj_lng

    x = gcj_lng
    y = gcj_lat
    z = math.hypot(x, y) + 0.00002 * math.sin(y * math.pi)
    theta = math.atan2(y, x) + 0.000003 * math.cos(x * math.pi)
    bd_lng = z * math.cos(theta) + 0.0065
    bd_lat = z * math.sin(theta) + 0.006
    return bd_lat, bd_lng


def bd2gcj(bd_lat, bd_lng):
    """
    百度转GCJ－2
    :param bd_lat:
    :param bd_lng:
    :return:
    """
    if out_of_china(bd_lat, bd_lng):
        return bd_lat, bd_lng

    x = bd_lng - 0.0065
    y = bd_lat - 0.006
    z = math.hypot(x, y) - 0.00002 * math.sin(y * math.pi)
    theta = math.atan2(y, x) - 0.000003 * math.cos(x * math.pi)
    gcj_lng = z * math.cos(theta)
    gcj_lat = z * math.sin(theta)
    return gcj_lat, gcj_lng


def wgs2bd(wgs_lat, wgs_lng):
    return gcj2bd(*wgs2gcj(wgs_lat, wgs_lng))


def bd2wgs(bd_lat, bd_lng):
    return gcj2wgs(*bd2gcj(bd_lat, bd_lng))



# -*- coding: utf-8 -*-
# created by restran on 2018/01/22
from __future__ import unicode_literals, absolute_import
from mountains import json
import unittest
import uuid
from datetime import datetime


class Test(unittest.TestCase):
    def setUp(self):
        pass

    def test_json_loads(self):
        a = json.loads('{"a": 123, "b": "456", "c": "中文"}')
        self.assertEqual(a['a'], 123)
        self.assertEqual(a['b'], '456')
        self.assertEqual(a['c'], '中文')

    def test_json_dumps(self):
        data = {
            'a': uuid.uuid4(),
            'b': datetime(year=2016, month=10, day=30, hour=12, minute=30, second=30)
        }
        a = json.dumps(data)
        a = json.loads(a)
        self.assertEqual(a['b'], '2016-10-30 12:30:30')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# created by restran on 2018/04/06
from __future__ import unicode_literals, absolute_import


def get_client_ip(request):
    """
    获取客户端的IP
    :param request:
    :return:
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    x_real_ip = request.META.get('HTTP_X_REAL_IP')
    if x_real_ip:
        ip = x_real_ip
    elif x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip



# -*- coding: utf-8 -*-
# Created by restran on 2017/7/27
from __future__ import unicode_literals

import sys

from future.utils import bytes_to_native_str
from setuptools import setup, find_packages

from mountains import __version__

kwargs = {
    'packages': find_packages(),
    # 还需要创建一个 MANIFEST.in 的文件，然后将这些数据也放在那里
    # package_data 添加了配置，Python2会报错
    'package_data': {}
}

install_requires = [
    'requests',
    'simplejson',
    'colorlog'
]

if sys.version_info < (3, 0):
    install_requires.append('futures')

kwargs['install_requires'] = install_requires
readme_file = 'README.md'
long_description = open(readme_file, 'rb').read()

setup(
    name='mountains',  # 文件名
    version=__version__,  # 版本(每次更新上传 pypi 需要修改)
    description="a util collection for python developing",
    long_description=bytes_to_native_str(long_description),  # 放README.md文件，方便在 pypi 页展示
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],  # Get strings from http://pypi.python.org/pypi?:action=list_classifiers
    keywords='python utils',  # 关键字
    author='restran',  # 用户名
    author_email='grestran@gmail.com',  # 邮箱
    url='https://github.com/restran/mountains',  # github上的地址
    license='MIT',  # 遵循的协议
    include_package_data=True,
    zip_safe=True,
    platforms='any',
    **kwargs
)



# -*- coding: utf-8 -*-
# Created by restran on 2018/9/10
from __future__ import unicode_literals, absolute_import

import csv

__all__ = ['read_csv', 'read_csv_as_dict', 'write_csv', 'write_csv_from_dict']


def read_csv(filepath_or_buffer, encoding=None, delimiter=',', quotechar='"'):
    """
    读 CSV 文件
    :param encoding: 设置编码，如果文件是UTF-8 with BOM，encoding 使用 utf-8-sig
    :param filepath_or_buffer:
    :param delimiter:
    :param quotechar:
    :return:
    """

    if isinstance(filepath_or_buffer, str):
        with open(filepath_or_buffer, 'r', encoding=encoding) as csv_file:
            reader = csv.reader(csv_file, delimiter=delimiter, quotechar=quotechar)
            for row in reader:
                yield row

    else:
        reader = csv.reader(filepath_or_buffer, delimiter=delimiter, quotechar=quotechar)
        for row in reader:
            yield row


def read_csv_as_dict(filepath_or_buffer, headers=None, encoding=None, delimiter=',', quotechar='"'):
    """

    :param filepath_or_buffer:
    :param headers: 首行字段列表
    :param encoding: 设置编码，如果文件是UTF-8 with BOM，encoding 使用 utf-8-sig
    :param delimiter: 字段分隔符
    :param quotechar:
    :return:
    """
    if isinstance(filepath_or_buffer, str):
        with open(filepath_or_buffer, 'r', encoding=encoding) as csv_file:
            reader = csv.DictReader(csv_file, fieldnames=headers,
                                    delimiter=delimiter, quotechar=quotechar)
            for row in reader:
                yield dict(row)
    else:
        reader = csv.DictReader(filepath_or_buffer, fieldnames=headers,
                                delimiter=delimiter, quotechar=quotechar)
        for row in reader:
            yield dict(row)


def write_csv(filepath_or_buffer=None, headers=None, data=None,
              encoding=None, delimiter=',', quotechar='"'):
    """
    写 CSV
    :param filepath_or_buffer:
    :param headers:
    :param data:
    :param encoding:
    :param delimiter:
    :param quotechar:
    :return:
    """
    if isinstance(filepath_or_buffer, str):
        with open(filepath_or_buffer, 'w', encoding=encoding) as csv_file:
            writer = csv.writer(csv_file, delimiter=delimiter, quotechar=quotechar)
            if headers is not None and isinstance(headers, list):
                writer.writerow(headers)

            writer.writerows(data)
    else:
        writer = csv.writer(filepath_or_buffer, delimiter=delimiter, quotechar=quotechar)
        if headers is not None and isinstance(headers, list):
            writer.writerow(headers)

        writer.writerows(data)


def write_csv_from_dict(filepath_or_buffer=None, headers=None, data=None,
                        write_header=True, encoding=None, delimiter=',',
                        quotechar='"'):
    """
    data 是 list 类型，每一项都是字典类型
    :param write_header:
    :param filepath_or_buffer:
    :param headers:
    :param data:
    :param encoding:
    :param delimiter:
    :param quotechar:
    :return:
    """
    if isinstance(filepath_or_buffer, str):
        with open(filepath_or_buffer, 'w', encoding=encoding) as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=headers, delimiter=delimiter,
                                    quotechar=quotechar)
            if write_header:
                writer.writeheader()
            writer.writerows(data)
    else:
        writer = csv.DictWriter(filepath_or_buffer, fieldnames=headers, delimiter=delimiter,
                                quotechar=quotechar)
        if write_header:
            writer.writeheader()
        writer.writerows(data)



# -*- coding: utf-8 -*-
# Created by restran on 2018/11/13
from __future__ import unicode_literals, absolute_import

import unittest

from mountains.http import random_agent, random_mobile_agent, random_wx_agent


class HTTPTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_random_agent(self):
        random_agent()
        random_agent('pc')
        random_agent('wexin')
        random_agent('mobile')
        random_wx_agent()
        random_mobile_agent()
        self.assertEqual(True, True)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/29
from __future__ import unicode_literals, absolute_import

import os
import sys
import types

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
PYPY = True if getattr(sys, 'pypy_version_info', None) else False

if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
    long_type = int
    from queue import Queue
    from io import BytesIO, StringIO
    from urllib.parse import urlencode, quote, \
        quote_plus, urlparse, urlunparse

    MAXSIZE = sys.maxsize
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str
    long_type = long
    from cStringIO import StringIO
    from io import BytesIO
    from urllib import urlencode, quote, quote_plus
    from urlparse import urlparse, urlunparse
    from Queue import Queue

# 当前项目所在路径
__base_path = os.path.dirname(os.path.abspath(__file__))


def iteritems(obj, **kwargs):
    """Use this only if compatibility with Python versions before 2.7 is
    required. Otherwise, prefer viewitems().
    """
    func = getattr(obj, "iteritems", None)
    if not func:
        func = obj.items
    return func(**kwargs)


def iterkeys(obj, **kwargs):
    """Use this only if compatibility with Python versions before 2.7 is
    required. Otherwise, prefer viewkeys().
    """
    func = getattr(obj, "iterkeys", None)
    if not func:
        func = obj.keys
    return func(**kwargs)


def itervalues(obj, **kwargs):
    """Use this only if compatibility with Python versions before 2.7 is
    required. Otherwise, prefer viewvalues().
    """
    func = getattr(obj, "itervalues", None)
    if not func:
        func = obj.values
    return func(**kwargs)


__all__ = [
    'PY2', 'PY3', 'PYPY', 'urlencode', 'quote', 'quote_plus',
    'urlparse', 'urlunparse', 'StringIO',
    'string_types', 'integer_types', 'class_types', 'text_type',
    'binary_type', 'long_type', 'BytesIO', '__base_path', 'Queue',
    'iteritems', 'iterkeys', 'itervalues'
]



# -*- coding: utf-8 -*-
# Created by restran on 2017/8/23
from __future__ import unicode_literals, absolute_import

import os
import shutil
from collections import deque

from .. import json
from ..datetime.converter import timestamp2datetime
from ..encoding import force_text, force_bytes


def read_dict(file_name, clear_none=False, encoding='utf-8'):
    """
    读取字典文件
    :param encoding:
    :param clear_none:
    :param file_name:
    :return:
    """
    with open(file_name, 'rb') as f:
        data = f.read()

    if encoding is not None:
        data = data.decode(encoding)

    line_list = data.splitlines()
    data = []
    i = 0
    for line in line_list:
        i += 1
        try:
            line = force_text(line).strip()
            data.append(line)
        except:
            print('read error line %s' % i)
    if clear_none:
        data = [t for t in data if t != '']
    data = deque(data)
    return data


def write_bytes_file(file_name, data):
    with open(file_name, 'wb') as f:
        f.write(force_bytes(data))


def read_bytes_file(file_name):
    with open(file_name, 'rb') as f:
        return f.read()


def write_file(file_name, data):
    """
    写文本文件
    :param file_name:
    :param data:
    :return:
    """
    with open(file_name, 'w') as f:
        f.write(data)


def read_file(file_name, encoding='utf-8'):
    """
    读文本文件
    :param encoding:
    :param file_name:
    :return:
    """
    with open(file_name, 'rb') as f:
        data = f.read()

    if encoding is not None:
        data = data.decode(encoding)

    return data


def read_json(file_name):
    try:
        return json.loads(read_bytes_file(file_name))
    except:
        return None


def write_json(file_name, data, indent=None,
               ensure_ascii=True, sort_keys=False, **kwargs):
    with open(file_name, 'wb') as f:
        try:
            data = json.dumps(data, indent=indent,
                              ensure_ascii=ensure_ascii,
                              sort_keys=sort_keys, **kwargs)
            f.write(force_bytes(data))
            return True
        except:
            return False


def get_file_size(file_path):
    """
    获取文件大小，返回的是字节
    :param file_path:
    :return:
    """
    return os.path.getsize(file_path)


def get_file_access_time(file_path):
    """
    获取文件访问时间，返回 datetime 类型
    :param file_path:
    :return:
    """
    return timestamp2datetime(os.path.getatime(file_path))


def get_file_create_time(file_path):
    """
    获取文件创建时间，返回 datetime 类型
    :param file_path:
    :return:
    """
    return timestamp2datetime(os.path.getctime(file_path))


def get_file_modify_time(file_path):
    """
    获取文件修改时间，返回 datetime 类型
    :param file_path:
    :return:
    """
    return timestamp2datetime(os.path.getmtime(file_path))


def copy_files(src_path, dst_path):
    """
    :param src_path:
    :param dst_path:
    :return:
    """
    abs_src_path = os.path.abspath(src_path)
    abs_dst_path = os.path.abspath(dst_path)
    # 遍历src_path目录下的所有文件
    for root, dirs, files in os.walk(src_path):
        try:
            for f in files:
                src_file_p = os.path.abspath(os.path.join(root, f))
                # 目标文件的完整路径
                dst_file_p = os.path.abspath(os.path.join(
                    abs_dst_path, src_file_p[len(abs_src_path) + 1:]))
                # 判断目标文件是否已存在，已存在则改名
                if os.path.exists(dst_file_p):
                    continue

                try:
                    # 判断目标文件所在的文件夹是否存在，不存在则递归创建文件夹
                    dst_p = os.path.dirname(dst_file_p)
                    if not os.path.exists(dst_p):
                        os.makedirs(dst_p)

                    # 移动文件
                    shutil.copy2(src_file_p, dst_file_p)
                except Exception as e:
                    print('move file error: {}'.format(e))

        except Exception as ex:
            print('error: {}'.format(ex))



# -*- coding: utf-8 -*-
# Created by restran on 2017/11/7
from __future__ import unicode_literals, absolute_import
from ..encoding import force_bytes, text_type, force_text
from .. import PY2, string_types


def fixed_length_split(s, width):
    """
    固定长度分割字符串
    :param s:
    :param width:
    :return:
    """
    # 使用正则的方法
    # import re
    # split = re.findall(r'.{%s}' % width, string)
    return [s[x: x + width] for x in range(0, len(s), width)]


def line_break(s, length=76):
    """
    将字符串分割成一行一行
    :param s:
    :param length:
    :return:
    """
    x = '\n'.join(s[pos:pos + length] for pos in range(0, len(s), length))
    return x


def is_empty(s):
    if s == '' or s == b'' or s is None or not isinstance(s, text_type):
        return True
    else:
        return False


def any_empty(*params):
    for s in params:
        if is_empty(s):
            return True
    else:
        return False


def bytes_2_printable_strings(data):
    data = force_bytes(data)
    result = ['', '']
    for c in data:
        if PY2:
            c = ord(c)

        if 32 <= c <= 126 or c in (9, 10, 13):
            if c == 9:
                c = 32
            elif c == 13:
                c = 10

            # 去掉连续的空格
            if c == 32 and result[-1] == ' ':
                continue
            # 去掉连续的换行
            elif c == 10 and result[-1] == '\n':
                continue

            result.append(chr(c))

    return ''.join(result)
